#<profile4xi profile = "00"/>
# -------------------------------------------------------------------------------------------------
#<program4xi program = "test_proxy_mp_xi4.py">
#<abstract4xi abstract = "Proxy Objects in Multiprocessing"/>
#<verify4xi verification = "11" target = "manager4xi.py"/>
#<contextualize4xi context = "syncmanager4xi.py; process4xi.py; pipe4xi.py; forkingpickler4xi.py; popen4xi.py; current_process4xi.py"/>
#<refer4xi reference = "DragonXi"/>


#<note4xi note = "                      
#
# Theory:      Multiprocessing
#
# Description: A proxy is an object which refers to a shared object which lives (presumably) in a different process. 
#              The shared object is said to be the referent of the proxy. 
#              Multiple proxy objects may have the same referent.
#
#              A proxy object has methods which invoke corresponding methods of its referent (although not every method of the referent will necessarily be available through the proxy). 
#              A proxy can usually be used in most of the same ways that its referent can
#
# class multiprocessing.Process([group[, target[, name[, args[, kwargs]]]]]) 
# 
#    Process objects represent activity that is run in a separate process. 
#    The Process class has equivalents of all the methods of threading.Thread.
#    
# constructor
#    should always be called with keyword arguments. 
#    
# group
#    should always be None; it exists solely for compatibility with Thread. 
#    
# target
#    is the callable object to be invoked by the run() method. 
#    It defaults to None, meaning nothing is called. 
#    
# name
#    is the process name
#    by default, a unique name is constructed of the form Process-N1:N2:...:Nk
#    where N1,N2,...,Nk is a sequence of integers whose length is determined by the generation of the process. 
#    
# args
#    is the argument tuple for the target invocation
#    
# kwargs
#    is a dictionary of keyword arguments for the target invocation. By default, no arguments are passed to target.
#    
# If a subclass overrides the constructor, it must make sure it invokes the base class constructor (Process.__init__()) before doing anything else to the process.
#    
# run() 
#    Method representing the process activity.
#    You may override this method in a subclass. 
#    The standard run() method invokes the callable object passed to the object constructor as the target argument, if any, 
#    with sequential and keyword arguments taken from the args and kwargs arguments, respectively.
#    
# start() 
#    Start the process activity.
#    This must be called at most once per process object. 
#    It arranges for the object run() method to be invoked in a separate process.
#    
# join([timeout]) 
#    Block the calling thread until the process whose join() method is called terminates or until the optional timeout occurs.
#    If timeout is None then there is no timeout.
#    A process can be joined many times.
#    A process cannot join itself because this would cause a deadlock. It is an error to attempt to join a process before it has been started.
#    
# name 
#    Return the process name.
#    
# Process.name = name 
#    Set the process name.
#    The name is a string used for identification purposes only. 
#    It has no semantics. 
#    Multiple processes may be given the same name. 
#    The initial name is set by the constructor.
#    
# is_alive() 
#    Return whether the process is alive.
#    Roughly, a process object is alive from the moment the start() method returns until the child process terminates.
#    
# daemon 
#    Return the process daemon flag., this is a boolean.
#    
# Process.daemon = daemonic 
#    Set the process daemon flag to the Boolean value daemonic. 
#    This must be called before start() is called.
#    The initial value is inherited from the creating process.
#    When a process exits, it attempts to terminate all of its daemonic child processes.
#    Note that a daemonic process is not allowed to create child processes. 
#    Otherwise a daemonic process would leave its children orphaned if it gets terminated when its parent process exits.
#        
# In addition process objects also support the following methods:
#    
# pid 
#    Return the process ID. Before the process is spawned, this will be None.
#    
# exitcode 
#    Return the child exit code. This will be None if the process has not yet terminated. A negative value -N indicates that the child was terminated by signal N.
#
# authkey 
#    Return the process authentication key (a byte string).
#    When multiprocessing is initialized the main process is assigned a random string using os.random().
#    When a Process object is created, it will inherit the authentication key of its parent process, although this may be changed using Process.authkey below.
#    See Authentication keys.
# 
# Process.authkey = authkey 
#    Set the process authentication key which must be a byte string.
# 
# terminate() 
#    Terminate the process. 
#    On Unix this is done using the SIGTERM signal; 
#    on Windows TerminateProcess is used. Note that exit handlers and finally clauses, etc., will not be executed.
#    Note that descendant processes of the process will not be terminated, they will simply become orphaned
#    Warning
#    If this method is used when the associated process is using a pipe or queue 
#    then the pipe or queue is liable to become corrupted and may become unusable by other process. 
#    Similarly, if the process has acquired a lock or semaphore etc. then terminating it is liable to cause other processes to deadlock.
#    Note that the start(), join(), is_alive() and get_exit_code() methods should only be called by the process that created the process object.
#
#"/> 
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'print_trace4xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
import sys
from ctypes import Structure, c_double
import threading
import _thread
#"
#/>


# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import Process, Queue, Pipe, Lock, Manager, Pool
from multiprocessing.sharedctypes import Value, Array
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "

from arrayfunction2xi import ArrayFunction2Xi #<--- note: this must be called (Array4Xi uses the class)
from lock2xi import Lock2Xi
from manager2xi import Manager2Xi 
from pipe2xi import Pipe2Xi
from pool2xi import Pool2Xi
from process2xi import Process2Xi
from queue2xi  import Queue2Xi
from value2xi import Value2Xi


from arrayfunction4xi import ArrayFunction4Xi #<--- note: this must be called (Array4Xi uses the class)
from lock4xi import Lock4Xi
from manager4xi import Manager4Xi 
from pipe4xi import Pipe4Xi
from pool4xi import Pool4Xi
from process4xi import Process4Xi

from print4xi import Print4Xi

from pickle4xi import pickle4xi
from pickle2a4xi import Pickle2a4Xi
from pickle2b4xi import Pickle2b4Xi
from pickle2c4xi import Pickle2c4Xi
from pickle2d4xi import Pickle2d4Xi
from pickle2e4xi import Pickle2e4Xi
from pickle2f4xi import Pickle2f4Xi
from pickle2g4xi import Pickle2g4Xi
from pickle2h4xi import Pickle2h4Xi
from pickle2i4xi import Pickle2i4Xi
from pickle2j4xi import Pickle2j4Xi
from pickle2k4xi import Pickle2k4Xi
from pickle2l4xi import Pickle2l4Xi
from pickle2m4xi import Pickle2m4Xi
from pickle2n4xi import Pickle2n4Xi
from pickle2o4xi import Pickle2o4Xi
from pickle2p4xi import Pickle2p4Xi
from pickle2q4xi import Pickle2q4Xi
from pickle2r4xi import Pickle2r4Xi
from pickle2s4xi import Pickle2s4Xi
from pickle2t4xi import Pickle2t4Xi
from pickle2u4xi import Pickle2u4Xi
from pickle2v4xi import Pickle2v4Xi
from pickle2w4xi import Pickle2w4Xi
from pickle2x4xi import Pickle2x4Xi
from pickle2y4xi import Pickle2y4Xi
from pickle2z4xi import Pickle2z4Xi
from pickle2_4xi import Pickle2_4Xi      
from queue4xi  import Queue4Xi
from time4xi import time4xi
from value4xi import Value4Xi

from xml_trace_sense4xi import xml_trace_sense4xi
from xml_trace_reason4xi import xml_trace_reason4xi
from xml_trace_ace4xi import xml_trace_ace4xi 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global t2xi 
global trace_object_count
global PickleKB4Xi

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
PickleKB4Xi = {
       'a': Pickle2a4Xi,
       'b': Pickle2b4Xi,
       'c': Pickle2c4Xi,
       'd': Pickle2d4Xi,
       'e': Pickle2e4Xi,
       'f': Pickle2f4Xi,
       'g': Pickle2g4Xi,
       'h': Pickle2h4Xi,
       'i': Pickle2i4Xi,
       'j': Pickle2j4Xi,
       'k': Pickle2k4Xi,
       'l': Pickle2l4Xi,
       'm': Pickle2m4Xi,
       'n': Pickle2n4Xi,
       'o': Pickle2o4Xi,
       'p': Pickle2p4Xi,
       'q': Pickle2q4Xi,
       'r': Pickle2r4Xi,
       's': Pickle2s4Xi,
       't': Pickle2t4Xi,
       'u': Pickle2u4Xi,
       'v': Pickle2v4Xi,
       'w': Pickle2w4Xi,
       'x': Pickle2x4Xi,
       'y': Pickle2y4Xi,
       'z': Pickle2z4Xi,
       '_': Pickle2_4Xi} 
#"
#"/>

# ----------------------------------------------------------------------------------------------------------
#<function4xi function = "set2trace4xi" usage = "setting trace using definitions in pickle4xi functions !
#">                      
# ----------------------------------------------------------------------------------------------------------    
def set2trace4xi():
    global t2xi 
    global trace_object_count
    global PickleKB4Xi

    # print("(Step 0A/33) OF SET2TRACE4XI - STARTING SETTING TRACE BASED ON TRACE SETTINGS IN MODULES PICKLE2x4XI.PY (x=a,b...) ", time4xi()) 
    # print("(Step 0B/33) OF SET2TRACE4XI - USING T2XI: ", t2xi)
    # <print4xi.Print4Xi object at 0x0000000003715080>    
    
    # ---------------------------------------------------------------------------------------------
    # A, B .... _ 
    # ---------------------------------------------------------------------------------------------
    keys2pickledb = sorted(PickleKB4Xi.keys())
    for key2pickledb in keys2pickledb:
        # -----------------------------------------------------------------------------------------    
        # LETTERS
        # -----------------------------------------------------------------------------------------     
        # print('=' *33)
        # print("STARTING LETTER : ", key2pickledb)
        # print('=' *33)
   
        dictionary2pickle = PickleKB4Xi[key2pickledb]        
        # print("STARTING DICTIONARY: ", dictionary2pickle)
        # print('~' *33) 
        # 
        keys2dictionary = sorted(dictionary2pickle.keys())
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
        for key2dictionary in keys2dictionary:
            # -------------------------------------------------------------------------------------    
            # DICTIONARY
            # -------------------------------------------------------------------------------------     
            # print('~' *33)        
            object2pickle = key2dictionary        
            # print("STARTING OBJECT: ", object2pickle)
            # print('~' *33)         
            # pickle4xi(object2pickle)
            # print('~' *33)                  
            # print("PICKLED object: ", object2pickle)
            # PICKLED object:  _C4Xi, 
            # ---------------------------------------------------------------------------------    
            # Mark those with NACK4XI to be traced !
            # ---------------------------------------------------------------------------------     
            if dictionary2pickle[key2dictionary] == 'NACK4XI':
                set_param2xi = True     
                key_param = object2pickle     
                t2xi.set_print_permission(key_param, set_param2xi)
                # print("(Step 2A/33) OF SET2TRACE4XI - MARKED TO BE TRACED OBJECT: ", object2pickle)                                
                # 
            else:
                pass  
            # ---------------------------------------------------------------------------------    
            # END OF DICTIONARY
            # ---------------------------------------------------------------------------------     
        # -----------------------------------------------------------------------------------------    
        # END OF LETTERS
        # -----------------------------------------------------------------------------------------     
        # print('~' *33)        
        # print("COMPLETED  DICTIONARY: ", dictionary2pickle)
        # print('~' *33) 
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------     
    # print('=' *33)
    # print("COMPLETED LETTER : ", key2pickledb)
    # print('=' *33)
    # ---------------------------------------------------------------------------------------------
    # 
    # ---------------------------------------------------------------------------------------------
    print('=' *33)
    print("(Step 33/33) OF SET2TRACE4XI - COMPLETED SETTING TRACE BASED ON NACK4XI/ACK4XI SETTINGS IN MODULES PICKLE2x4XI.PY (x=a,b...) ", time4xi()) 
    print('=' *33)
#</function4xi> 

       
# -------------------------------------------------------------------------------------------------
#<function4xi function = "deny2trace4xi" usage = "
#   deny trace using pickle4xi functions !
#">                      
# -------------------------------------------------------------------------------------------------    
def deny2trace4xi():
    global t2xi 
    global trace_object_count
    global PickleKB4Xi

    print("(Step 01A/33) of DENY2TRACE4XI: STARTING DENYING TRACE IN MODULES PICKLE2x4XI.PY (x=a,b...) ", time4xi()) 
    print("(Step 01B/33) OF DENY2TRACE4XI - USING T2XI: ", t2xi)
    # <print4xi.Print4Xi object at 0x0000000003715080>
    
    # ---------------------------------------------------------------------------------------------
    # A, B .... _ 
    # ---------------------------------------------------------------------------------------------
    keys2pickledb = sorted(PickleKB4Xi.keys())
    for key2pickledb in keys2pickledb:
        # -----------------------------------------------------------------------------------------    
        # LETTERS
        # -----------------------------------------------------------------------------------------     
        # print("(Step 02A/33) OF DENY2TRACE4XI - STARTING LETTER : ", key2pickledb)
        dictionary2pickle = PickleKB4Xi[key2pickledb]        
        # print("(Step 02B/33) OF DENY2TRACE4XI - STARTING DICTIONARY: ", dictionary2pickle)
        keys2dictionary = sorted(dictionary2pickle.keys())
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
        for key2dictionary in keys2dictionary:
   
            # -------------------------------------------------------------------------------------    
            # DICTIONARY
   
            object2pickle = key2dictionary        
            # print("(Step 02C/33) OF DENY2TRACE4XI - STARTING OBJECT: ", object2pickle)
            # pickle4xi(object2pickle)        
            # print("STEP: PICKLED object: ", object2pickle)
            # PICKLED object:  _C4Xi, 
            
            # ---------------------------------------------------------------------------------    
            # ACK4XI = tested .i.e. NOT to be traced! 
            # ---------------------------------------------------------------------------------     
            set_param2xi = 'ACK4XI'     
            key_param = object2pickle     
            t2xi.set_print_permission(key_param, set_param2xi)
            # print("(Step 02D/33) OF DENY2TRACE4XI - MARKED TESTED AND NOT to be traced: ", object2pickle)
            
            dictionary2pickle[key2dictionary] = 'ACK4XI' 
                    
        
            # ---------------------------------------------------------------------------------    
            # END OF DICTIONARY
            # ---------------------------------------------------------------------------------     
            # print('~' *33)              
            # print("(Step 02Z/33) OF DENY2TRACE4XI *** FAILED - ERROR IN PICKLING OBJECT - NOT TO BE TRACED: ", object2pickle)
            # print('~' *33) 
        # -----------------------------------------------------------------------------------------    
        # END OF LETTERS
        # -----------------------------------------------------------------------------------------     
        # print("(Step 02E/33) OF DENY2TRACE4XI - COMPLETED  DICTIONARY: ", dictionary2pickle)
        # 
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------     
    # print("(Step 02F/33) OF DENY2TRACE4XI - COMPLETED LETTER : ", key2pickledb)
    # _, a ... z            
    print("(Step 033/33) OF SET2TRACE4XI - COMPLETED DENYING TRACE IN MODULES PICKLE2x4XI.PY (x=a,b...) ", time4xi()) 
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "tailor2trace4xi" usage = "
#   tailors trace 
#">                      
# -------------------------------------------------------------------------------------------------    
def tailor2trace4xi():
    global trace_object_count
    global PickleKB4Xi 
    global t2xi
 
    print("(Step 001A/33) OF TAILOR2TRACE4XI - USING T2XI: ", t2xi)
    # <print4xi.Print4Xi object at 0x0000000003715080> 
                         
    # ---------------------------------------------------------------------------------------------
    # Settings to tailor trace
    # ---------------------------------------------------------------------------------------------
    #<set4xi setting = "
    KB4Tailoring = {
                'ApplyResult4Xi': 'ACK4TRACE',
    
                'Client4Xi': 'ACK4TRACE',
             
                'dump': 'NACK4TRACE',                                      
                'duplicate': 'NACK4TRACE', 

                'freeze_support': 'NACK4TRACE',    
                                                
                'get_command_line': 'NACK4TRACE',                                        
                'get_preparation_data4xi': 'NACK4TRACE',
                

                'get_run_pool4xi': 'ACK4TRACE',                      # constant_pool4xi.py
                'get_close_pool4xi': 'ACK4TRACE',
                'get_terminate_pool4xi': 'ACK4TRACE',
                'get_exitpriority_pool4xi': 'ACK4TRACE',
                'get_job_counter_pool4xi': 'ACK4TRACE',
                          
                                                  
                'is_forking': 'NACK4TRACE',                               
                
                'init_run_pool4xi': 'ACK4TRACE',                     # constant_pool4xi.py
                'init_close_pool4xi': 'ACK4TRACE',
                'init_terminate_pool4xi': 'ACK4TRACE',
                'init_exitpriority_pool4xi': 'ACK4TRACE',
                'init_job_counter_pool4xi': 'ACK4TRACE',
           
                'main': 'NACK4TRACE',                                        
                'MainProcess4Xi': 'ACK4TRACE',                                                                  
                'Manager4Xi': 'ACK4TRACE',              
                
                'next_job_counter_pool4xi': 'ACK4TRACE',
                
                'PipeListener4Xi': 'ACK4TRACE',
                'Pool4Xi': 'ACK4TRACE',                
                'Process4Xi': 'ACK4TRACE',                               
                        
                'Listener4Xi': 'ACK4TRACE',          
                'Lock4Xi':'ACK4TRACE',                                   
             
                'Pipe4Xi': 'ACK4TRACE',
                'PipeClient4Xi': 'ACK4TRACE',                                
                'Popen4Xi': 'ACK4TRACE',       
                                       
                'Queue4Xi': 'ACK4TRACE',       
                            
                'reduce_connection': 'NACK4TRACE',            
                       
                'SemLock4Xi': 'ACK4TRACE',                            
                'Server4Xi': 'ACK4TRACE',
                'SyncManager4Xi': 'ACK4TRACE',         
                
                'set_run_pool4xi': 'ACK4TRACE',                      # constant_pool4xi.py
                'set_close_pool4xi': 'ACK4TRACE',
                'set_terminate_pool4xi': 'ACK4TRACE',
                'set_exitpriority_pool4xi': 'ACK4TRACE',
                'set_job_counter_pool4xi': 'ACK4TRACE',
                          
                '_MainProcess4Xi': 'ACK4TRACE'}                         
                
    default3trace = 'NACK4TRACE' # we do NOT want trace if object is not found! 
    #"/>
    
    # ---------------------------------------------------------------------------------------------
    # PROCESS KB4Tailoring
    # ---------------------------------------------------------------------------------------------
    keys2tailor = sorted(KB4Tailoring.keys())
    for key2tailor in keys2tailor:
        # ---------------------------------------------------------------------------------    
        # NACK4XI = NOT TESTED .I.E. TO BE TRACED! 
        # ---------------------------------------------------------------------------------     
        set_param2xi = 'NACK4XI'     
        key_param = key2tailor    
        t2xi.set_print_permission(key_param, set_param2xi)
        print("(Step 002A/33) OF TAILOR_TRACE4XI - MARKED TO BE TESTED: ", key2tailor)
        # 
        
    # -----------------------------------------------------------------------------------------    
    # EOF KEYS
    # -----------------------------------------------------------------------------------------     
    print("(Step 0033/33) OF TAILOR_TRACE4XI - COMPLETED  TAILORING: ", time4xi())
    #

    
#</function4xi>       
             
  
# -------------------------------------------------------------------------------------------------
#<function4xi function = "print_trace4xi" usage = "
# trace to XML files
#">                       
# -------------------------------------------------------------------------------------------------    
def print_trace4xi(t2xi):
       

    # -------------------------------------------------------------------------------------------------
    # SENSE                        
    # -------------------------------------------------------------------------------------------------  
   
    context2sense = {
     'file_path4xi': '..\\..\\xml\\sense-proxy-mp-test4xi.xml',
     'name_space4xi': '   xmlns="http://www.dragonxi2.com/sense/sw4xi/python32/owl/multiprocessing4xi.owl#"\n',
     'xml_base4xi': '   xml:base="http://www.yourdragonxi.org/web/lib/owl/multiprocessing4xi.owl">\n',
     'ontology_about4xi': '   <owl:Ontology rdf:about="Sensing of Multiprocessing4Xi">\n',
     'ontology_comment4xi': '      <rdfs:comment xml:lang="en">Classes, methods and messages with context.</rdfs:comment>\n',
     'class0_id4xi': '   <owl:Class rdf:ID="Dragon4Xi#000000">\n',
     'class0_comment4xi': '      <rdfs:comment xml:lang="en">Sensing Proxies in Multiprocessing.</rdfs:comment>\n',
     'object_property_rdf4xi': '   <owl:ObjectProperty rdf:ID="sensed#000000">\n',
     'object_property_comment4xi': '       <rdfs:comment xml:lang="en">Classes, Methods and Messages for Proxies in Multiprocessing.</rdfs:comment>\n',
     'debug4xi': True}
      
    xml_trace_sense4xi(t2xi, context2sense)

    # -------------------------------------------------------------------------------------------------
    # REASON                         
    # -------------------------------------------------------------------------------------------------  
        
    context2reason = {
     'file_path4xi': '..\\..\\xml\\reason-proxy-mp-test4xi.xml',
     'name_space4xi': '   xmlns="http://www.dragonxi4.com/reason/sw4xi/python32/owl/multiprocessing4xi.owl#"\n',
     'xml_base4xi': '   xml:base="http://www.dragonxi4.com/reason/sw4xi/python32/owl/multiprocessing4xi.owl">\n',
     'ontology_about4xi': '   <owl:Ontology rdf:about="Reasoning of Multiprocessing4Xi">\n',
     'ontology_comment4xi': '      <rdfs:comment xml:lang="en">Operations</rdfs:comment>\n',
     'operation_comment4xi': '      <rdfs:comment xml:lang="en">Operations for Proxies in Multiprocessing.</rdfs:comment>\n',
     'class0_id4xi': '   <owl:Class rdf:ID="reasoned#000000">\n',
     'class0_comment4xi': '      <rdfs:comment xml:lang="en">Operations Proxies in Multiprocessing.</rdfs:comment>\n',
     'object_property_rdf4xi': '   <owl:ObjectProperty rdf:ID="reason#000000">\n',
     'object_property_comment4xi': '       <rdfs:comment xml:lang="en">reason</rdfs:comment>\n',
     'debug4xi': True}
       
    xml_trace_reason4xi(t2xi, context2reason)

    # -------------------------------------------------------------------------------------------------
    # ACE                         
    # -------------------------------------------------------------------------------------------------  
       
    context2ace = {
     'file_path4xi': '..\\..\\xml\\ace-proxy-mp-test4xi.xml',
     'name_space4xi': '   xmlns="http://www.dragonxi8.com/ace/sw4xi/python32/owl/multiprocessing4xi.owl#"\n',
     'xml_base4xi': '   xml:base="http://www.dragonxi8.com/ace/sw4xi/python32/owl/multiprocessing4xi.owl">\n',
     'ontology_about4xi': '   <owl:Ontology rdf:about="Acing of Multiprocessing4Xi">\n',
     'ontology_comment4xi': '      <rdfs:comment xml:lang="en">Objects and values for Proxies in Multiprocessing.</rdfs:comment>\n',
     'class0_id4xi': '   <owl:Class rdf:ID="ACE4Xi#0000000">\n',
     'class0_comment4xi': '      <rdfs:comment xml:lang="en">Objects and values for Proxies in Multiprocessing.</rdfs:comment>\n',
     'object_property_rdf4xi': '   <owl:ObjectProperty rdf:ID="ace#0000000">\n',
     'object_property_comment4xi': '       <rdfs:comment xml:lang="en">000000</rdfs:comment>\n',
     'debug4xi': True}   
    
    xml_trace_ace4xi(t2xi, context2ace)


    # -------------------------------------------------------------------------------------------------
    # end of print_trace4xi                        
    # -------------------------------------------------------------------------------------------------
#</function4xi>
                                
# -------------------------------------------------------------------------------------------------
#<class4xi class = "Point4Xi" base = "Structure" child = "None" synopsis = "2-dimention points" usage = "testing 2-dimention points with pipes"
#>
# -------------------------------------------------------------------------------------------------
class Point4Xi(Structure):
    _fields_ = [('x', c_double), ('y', c_double)]

#</class4xi>

  
# -------------------------------------------------------------------------------------------------
#<function4xi function = "modify2xi" usage = "
# modifies 2-dimension
#"> 
# -------------------------------------------------------------------------------------------------    
def modify2xi(n_param, x_param, s_param, A_param):
    n_xi2 = n_param
    x_xi2 = x_param
    s_xi2 = s_param
    A_xi2 = A_param

    n_xi2.value **= 2
    x_xi2.value **= 2
    s_xi2.value = s_xi2.value.upper()
    
    for a_xi2 in A_xi2:
        a_xi2.x **= 2
        a_xi2.y **= 2
#</function4xi>
 
# -------------------------------------------------------------------------------------------------
#<class4xi class = "ThreeDimension3Xi" base = "Structure" child = "None" synopsis = "defines 3-dimension structure" usage = "testing 3-dimention points with pipes"
#>
# -------------------------------------------------------------------------------------------------
class ThreeDimension3Xi(Structure):
    _fields_ = [('x', c_double), ('y', c_double), ('z', c_double)]   
#</class4xi>

         
# -------------------------------------------------------------------------------------------------
#<function4xi function = "modify3xi" usage = "
# modifies 3-dimension structure
#"> 
# -------------------------------------------------------------------------------------------------    
def modify3xi(n_param, x_param, s_param, A_param):
    n_xi3 = n_param
    x_xi3 = x_param
    s_xi3 = s_param
    A_xi3 = A_param

    n_xi3.value **= 3
    x_xi3.value **= 3
    s_xi3.value = s_xi3.value.upper() 
    
    for a in A_xi3:
        a.x **= 3
        a.y **= 3
        a.z **= 3
#</function4xi>
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "sense" usage = "
# senses pipes
#"> 
# -------------------------------------------------------------------------------------------------
def sense(target_param, queue_param, pipe_param,lock_param,dict_param,list_param):
    target_xi2 = target_param
    queue_xi2 = queue_param
    pipe_publisher_channel_xi2 = pipe_param
    lock_xi2 = lock_param
    dict_xi2 = dict_param
    list_xi2 = list_param
    
    
    print("SENSE: ", target_xi2)
    
    # ---------------------------------------------------------------------------------------------
    # put a message to the queue
    # ---------------------------------------------------------------------------------------------
     
    queue_xi2.put(['DragonXi2_00', target_xi2, 'sensing!']) # puts to its queue
    
    # ---------------------------------------------------------------------------------------------
    # reserve the pipe using the lock, send messages and release the pipe
    # ---------------------------------------------------------------------------------------------
     
    lock_xi2.acquire()

    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_01', 'BIO ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_02', 'CO2 ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_03', 'Coal ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_04', 'Diesel ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_05', 'Electricity ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_06', 'Fusion ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_07', 'Gas ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_08', 'Geo ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_09', 'Hydrogen ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_10', 'Lithium ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_11', 'Nuclear Power ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_12', 'Ocean ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_13', 'Oil ',target_xi2]) 
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_14', 'Solar ',target_xi2])
    pipe_publisher_channel_xi2.send(['DragonXi2_Sensing_15', 'Wind ',target_xi2])
    
    dict_xi2[1] = "BIO"
    dict_xi2[2] = "CO2"
    dict_xi2[3] = "Coal"
    dict_xi2[4] = "Diesel"
    dict_xi2[5] = "Electricity"
    dict_xi2[6] = "Fusion"
    dict_xi2[7] = "Gas"
    dict_xi2[8] = "Geo"
    dict_xi2[9] = "Hydrogen"
    dict_xi2[10] = "Lithium"
    dict_xi2[11] = "Nuclear Power"
    dict_xi2[12] = "Ocean"
    dict_xi2[13] = "Oil"
    dict_xi2[14] = "Solar"
    dict_xi2[15] = "Wind"
    

    lock_xi2.release()       
#</function4xi>
     
    
# -------------------------------------------------------------------------------------------------
#<function4xi function = "reason" usage = "
# reasons pipes
#"> 
# ------------------------------------------------------------------------------------------------- 
def reason(target_param, queue_param, pipe_param,lock_param,dict_param,list_param):

    target_xi4 = target_param
    queue_xi4 = queue_param
    pipe_publisher_channel_xi4 = pipe_param
    lock_xi4 = lock_param
    dict_xi4 = dict_param
    list_xi8 = list_param
    
    print("REASON: ", target_xi4)
    
    # ---------------------------------------------------------------------------------------------
    # put a message to the queue
    # ---------------------------------------------------------------------------------------------
     
    queue_xi4.put(['DragonXi4_00', target_xi4, 'portfolio must be sustainable!']) # puts to its queue 
    
    # ---------------------------------------------------------------------------------------------
    # reserve the pipe using the lock, send messages and release the pipe
    # ---------------------------------------------------------------------------------------------
     
    lock_xi4.acquire()
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_01', 'BIO ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_02', 'Geo ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_03', 'Hydrogen ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_04', 'Nuclear Power ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_05', 'Ocean ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_06', 'Solar ',target_xi4])
    pipe_publisher_channel_xi4.send(['DragonXi4_Reasoning_07', 'Wind ',target_xi4])    

    dict_xi4[1] = "BIO"
    dict_xi4[2] = "Geo"
    dict_xi4[3] = "Hydrogen"
    dict_xi4[4] = "Nuclear Power"
    dict_xi4[5] = "Ocean"
    dict_xi4[6] = "Solar"
    dict_xi4[7] = "Wind"
    
    lock_xi4.release()
#</function4xi>
         
# -------------------------------------------------------------------------------------------------
#<function4xi function = "ace" usage = "
# aces pipes
#"> 
# -------------------------------------------------------------------------------------------------
def ace(target_param, queue_param, pipe_param,lock_param,dict_param,list_param):
    
    # ---------------------------------------------------------------------------------------------
    # 
    # ---------------------------------------------------------------------------------------------
     
    target_xi8 = target_param
    queue_xi8 = queue_param
    pipe_publisher_channel_xi8 = pipe_param
    lock_xi8 = lock_param
    dict_xi8 = dict_param
    list_xi8 = list_param
    
    print("ACE: ", target_xi8)
    
    # ---------------------------------------------------------------------------------------------
    # put a message to the queue
    # ---------------------------------------------------------------------------------------------
     
    queue_xi8.put(['DragonXi8_00', target_xi8, 'to be aced!']) # puts to its queue       
    
    # ---------------------------------------------------------------------------------------------
    # reserve the pipe using the lock, send messages and release the pipe
    # ---------------------------------------------------------------------------------------------
     
    lock_xi8.acquire()
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_01', 'BIO ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_02', 'Hydrogen ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_03', 'Nuclear Power ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_04', 'Solar ',target_xi8])
    pipe_publisher_channel_xi8.send(['DragonXi8_Ace_05', 'Wind ',target_xi8])    

    dict_xi8[1] = "BIO"
    dict_xi8[2] = "Hydrogen"
    dict_xi8[3] = "Nuclear Power"
    dict_xi8[4] = "Solar"
    dict_xi8[5] = "Wind"
    
    lock_xi8.release()
#</function4xi>
     
# -------------------------------------------------------------------------------------------------
#<function4xi function = "expxi2" usage = "
#   exponent 2 pool worker
#"> 
# -------------------------------------------------------------------------------------------------
def expxi2(x_param):
    xi2 = x_param
    return xi2*xi2
#</function4xi>
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "expxi4" usage = "
#   exponent 4 pool worker
#"> 
# -------------------------------------------------------------------------------------------------
def expxi4(x_param):
    xi4 = x_param
    return xi4*xi4*xi4*xi4
#</function4xi>
     
# -------------------------------------------------------------------------------------------------
#<function4xi function = "expxi8" usage = "
#   exponent 8 pool worker
#"> 
# -------------------------------------------------------------------------------------------------
def expxi8(x_param):
    xi8 = x_param
    return xi8*xi8*xi8*xi8*xi8*xi8*xi8*xi8         
#</function4xi>
     
    
# -------------------------------------------------------------------------------------------------
#<function4xi function = "phase1" usage = "
#   using Proxy Objects in standard Multiprocessing 
#"> 
# -------------------------------------------------------------------------------------------------
def phase1():
    
    print('=' *33)
    print("STARTING PHASE 1!")
    print('=' *33)   
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 1/12) create multiprocessing queues for sensing, reasoning and acing
    # ---------------------------------------------------------------------------------------------
     
    queue2xi = Queue()
    queue4xi = Queue()
    queue8xi = Queue()
    
    print('=' *33)
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 1/12): CREATED MULTIPROCESSING QUEUES!")
    print('=' *33)   
        
    # ---------------------------------------------------------------------------------------------
    # (STEP 2/12): create multiprocessing pipes with consumer and publisher ends
    # ---------------------------------------------------------------------------------------------
         
    pipe2xi_consumer_channel, pipe2xi_publisher_channel = Pipe()
    pipe4xi_consumer_channel, pipe4xi_publisher_channel = Pipe()
    pipe8xi_consumer_channel, pipe8xi_publisher_channel = Pipe()
    
    print('=' *33)
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 2/12): CREATED MULTIPROCESSING PIPES WITH CONSUMER AND PUBLISHER ENDS!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 3/12): create three locks
    # ---------------------------------------------------------------------------------------------
       
    lock_xi2 = Lock()
    lock_xi4 = Lock()
    lock_xi8 = Lock()    
    
    print('=' *33)
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 3/12): CREATED THREE LOCKS!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 4/12): create three managers
    # ---------------------------------------------------------------------------------------------
           
    manager2xi = Manager()
    manager4xi = Manager()
    manager8xi = Manager()
    
    print('=' *33)
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 4/12): CREATED THREE MANAGERS!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 5/12): create three dictionaries using each manager
    # ---------------------------------------------------------------------------------------------
          
    dict2xi = manager2xi.dict()
    dict4xi = manager4xi.dict()
    dict8xi = manager8xi.dict()
    
    print('=' *33)
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 5/12): CREATED THREE DICTIONARIES USING EACH MANAGER!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 6/12): create three lists using each manager
    # ---------------------------------------------------------------------------------------------
          
    list2xi = manager2xi.list(range(15))
    list4xi = manager4xi.list(range(7))
    list8xi = manager8xi.list(range(5))
    
    print('=' *33)
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 6/12): CREATED THREE LISTS USING EACH MANAGER!")
    print('=' *33)   
         
        
    # ---------------------------------------------------------------------------------------------
    # (STEP 7/12): SENSE IT!
    #        create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
    
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 7/12): ~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~")
    process2xi = Process(target=sense, args=('Energy ',queue2xi, pipe2xi_publisher_channel,lock_xi2,dict2xi,list2xi))
    process2xi.start()
    
    print ("PROCESS2XI ALIVE STATUS: ", process2xi.is_alive())       # check whether the process is alive
    print ("PROCESS2XI NAME: ", process2xi.name)                     # the name of the process
    print ("PROCESS2XI PID: ", process2xi.ident)                     # pid = ident of the process
    print ("PROCESS2XI AUTHENTICATION KEY: ", process2xi.authkey)    # authorization key given by the system
    
    print (queue2xi.get())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    
    print("\n DICTIONARY CREATED BY MANAGER2XI AND UPDATED BY PROCESS2XI:")
    print (dict2xi)
    print ("DICT2XI PRESENTATION: ",repr(dict2xi))                   # proxy: prints presentation
    #print ("dict2xi Referent: ", dict2xi.__str__())                 # proxy: return the representation of the referent

    print("\n LIST FOR MANAGER2XI AND PROCESS2XI:")
    print (list2xi)
    # print (list2xi._call_method('__len__'))                        # failed: should call and return the result of a method of the proxy s referent 
    
    process2xi.join()
    print ("PROCESS2XI EXIT CODE AFTER JOIN : ", process2xi.exitcode) # see how the process did exit
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 8/12): REASON IT!
    #             create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
     
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 8/12): ~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~")
    process4xi = Process(target=reason, args=('Energy ',queue4xi, pipe4xi_publisher_channel,lock_xi4,dict4xi,list8xi))
    process4xi.start()
    
       
    print ("PROCESS4XI ALIVE STATUS: ", process4xi.is_alive())       # check whether the process is alive
    print ("PROCESS4XI NAME: ", process4xi.name)                     # the name of the process
    print ("PROCESS4XI PID: ", process4xi.ident)                     # pid = ident of the process
    print ("PROCESS4XI AUTHENTICATION KEY: ", process4xi.authkey)    # authorization key given by the system
    
    
    print (queue4xi.get())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    
    print("\n DICTIONARY CREATED BY MANAGER4XI AND UPDATED BY PROCESS4XI:")
    print (dict4xi)
    print ("DICT4XI PRESENTATION: ",repr(dict4xi))                   # proxy: prints presentation
    
    print("\n LIST FOR MANAGER4XI AND PROCESS4XI:")
    print (list4xi)
    
    process4xi.join()
    print ("PROCESS4XI EXIT CODE AFTER JOIN : ", process4xi.exitcode) # see how the process did exit
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 9/12): ACE IT!
    #            create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
     
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 9/12): ~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~")
    process8xi = Process(target=ace, args=('Energy ',queue8xi, pipe8xi_publisher_channel,lock_xi8,dict8xi,list8xi))
    process8xi.start()
    
           
    print ("PROCESS8XI ALIVE STATUS: ", process8xi.is_alive())       # check whether the process is alive
    print ("PROCESS8XI NAME: ", process8xi.name)                     # the name of the process
    print ("PROCESS8XI PID: ", process8xi.ident)                     # pid = ident of the process
    print ("PROCESS8XI AUTHENTICATION KEY: ", process8xi.authkey)    # authorization key given by the system
    
    
    print (queue8xi.get())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    
    print("\n DICTIONARY CREATED BY MANAGER8XI AND UPDATED BY PROCESS8XI:")
    print (dict8xi)
    print ("DICT8XI PRESENTATION: ",repr(dict8xi))                   # proxy: prints presentation
    
    print("\n LIST FOR MANAGER8XI AND PROCESS8XI:")
    print (list8xi)
    
    process8xi.join()
    print ("PROCESS8XI EXIT CODE AFTER JOIN : ", process8xi.exitcode) # see how the process did exit
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 10/12): create three pools with worker processes 
    # ---------------------------------------------------------------------------------------------
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 10/12): CREATING THREE POOLS WITH WORKER PROCESSES")
            
    pool2xi = Pool(processes=2)
    pool4xi = Pool(processes=4)
    pool8xi = Pool(processes=8)

    result2xi = pool2xi.apply_async(expxi2, (2,))
    print ("\n (STEP 10/12a): RESULT FROM POOL2XI WORKERS: ",result2xi.get(timeout=1))    
    
    result4xi = pool4xi.apply_async(expxi4, (4,))
    print ("\n (STEP 10/12b): RESULT FROM POOL4XI WORKERS: ",result4xi.get(timeout=1))
    
    result8xi = pool8xi.apply_async(expxi8, (8,))
    print ("\n (STEP 10/12c): RESULT FROM POOL8XI WORKERS: ",result8xi.get(timeout=1))
    
    
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 11/12): explore modifying ctypes objects in shared memory 
    # ---------------------------------------------------------------------------------------------
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 11/12): EXPLORING MODIFYING CTYPES OBJECTS IN SHARED MEMORY")
              
    lock_xi2 = Lock()

    n_xi2 = Value('i', 7)
    x_xi2 = Value(c_double, 1.0/3.0, lock=lock_xi2)
    s_xi2 = Array('c', 'yourDragonXi', lock=lock_xi2)
    A_xi2 = Array(Point4Xi, [(1.875,-6.25), (-5.75,2.0), (2.375,9.5)], lock=lock_xi2)

    # multiprocessing.Array(typecode_or_type, size_or_initializer, *, lock=True)
    #
    # Return a ctypes array allocated from shared memory. By default the return value is actually a synchronized wrapper for the array.
    # typecode_or_type
    #    determines the type of the elements of the returned array: it is either a ctypes type or a one character typecode of the kind used by the array module. 
    # size_or_initializer
    #    If size_or_initializer is an integer, then it determines the length of the array, and the array will be initially zeroed. 
    #    Otherwise, size_or_initializer is a sequence which is used to initialize the array and whose length determines the length of the array.
    # lock
    #    If lock is True (the default) then a new lock object is created to synchronize access to the value. 
    #    If lock is a Lock or RLock object then that will be used to synchronize access to the value. 
    #    If lock is False then access to the returned object will not be automatically protected by a lock, so it will not necessarily be process-safe
    #
    # Notes
    #    lock is a keyword only argument
    #    an array of ctypes.c_char has value and rawvalue attributes which allow one to use it to store and retrieve strings
    #

    # ---------------------------------------------------------------------------------------------
    #
    # ---------------------------------------------------------------------------------------------
    
    process2ctypes = Process(target=modify2xi, args=(n_xi2, x_xi2, s_xi2, A_xi2))
    process2ctypes.start()
    process2ctypes.join()

    print ("(STEP 11/12a) n_xi2.value: ",n_xi2.value)
    print ("(STEP 11/12b) x_xi2.value: ",x_xi2.value)
    print ("(STEP 11/12c) s_xi2.value: ",s_xi2.value)
    print ([(a.x, a.y) for a in A_xi2])
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 12/12): explore with 3D coordinates
    # ---------------------------------------------------------------------------------------------
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 12/12): EXPLORING WITH 3D COORDINATES")
        
    lock_xi3 = Lock()
    n_xi3 = Value('i', 8)
    x_xi3 = Value(c_double, 1.0/3.0, lock=lock_xi3)
    s_xi3 = Array('c', 'DragonXi3D', lock=lock_xi3)
    A_xi3 = Array(ThreeDimension3Xi, [(1.875,-6.25,2.00), (-5.75,2.0,3.00), (2.375,9.5,4.0)], lock=lock_xi3)
    
    process3xyz = Process(target=modify3xi, args=(n_xi3, x_xi3, s_xi3, A_xi3))
    process3xyz.start()
    process3xyz.join()
    
    print ("(STEP 12/12a) n_xi3.value: ",n_xi3.value)
    print ("(STEP 12/12b) x_xi3.value: ",x_xi3.value)
    print ("(STEP 12/12c) s_xi3.value: ",s_xi3.value)
    print ([(a.x, a.y, a.z) for a in A_xi3])
    
    print('=' *33)
    print("COMPLETED PHASE 1 - ORIGINAL MULTIPROCESSING!")
    print('=' *33)   

#</function4xi>
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "phase2" usage = "
#   using Proxy Objects in Multiprocessing2Xi 
#"> 
# -------------------------------------------------------------------------------------------------
def phase2():
    
    print('=' *33)
    print("STARTING PHASE 2!")
    print('=' *33)   
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 1/12) create multiprocessing queues for sensing, reasoning and acing
    # ---------------------------------------------------------------------------------------------
     
    queue2xi = Queue2Xi()
    queue4xi = Queue2Xi()
    queue8xi = Queue2Xi()
    
    print('=' *33)
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 1/12): CREATED MULTIPROCESSING QUEUES!")
    print('=' *33)   
        
    # ---------------------------------------------------------------------------------------------
    # (STEP 2/12): create multiprocessing pipes with consumer and publisher ends
    # ---------------------------------------------------------------------------------------------
         
    pipe2xi_consumer_channel, pipe2xi_publisher_channel = Pipe2Xi()
    pipe4xi_consumer_channel, pipe4xi_publisher_channel = Pipe2Xi()
    pipe8xi_consumer_channel, pipe8xi_publisher_channel = Pipe2Xi()
    
    print('=' *33)
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 2/12): CREATED MULTIPROCESSING PIPES WITH CONSUMER AND PUBLISHER ENDS!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 3/12): create three locks
    # ---------------------------------------------------------------------------------------------
       
    lock_xi2 = Lock2Xi()
    lock_xi4 = Lock2Xi()
    lock_xi8 = Lock2Xi()    
    
    print('=' *33)
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 3/12): CREATED THREE LOCKS!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 4/12): create three managers
    # ---------------------------------------------------------------------------------------------
           
    manager2xi = Manager2Xi()
    manager4xi = Manager2Xi()
    manager8xi = Manager2Xi()
    
    print('=' *33)
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 4/12): CREATED THREE MANAGERS!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 5/12): create three dictionaries using each manager
    # ---------------------------------------------------------------------------------------------
          
    dict2xi = manager2xi.dict()
    dict4xi = manager4xi.dict()
    dict8xi = manager8xi.dict()
    
    print('=' *33)
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 5/12): CREATED THREE DICTIONARIES USING EACH MANAGER!")
    print('=' *33)   
         
    # ---------------------------------------------------------------------------------------------
    # (STEP 6/12): create three lists using each manager
    # ---------------------------------------------------------------------------------------------
          
    list2xi = manager2xi.list(range(15))
    list4xi = manager4xi.list(range(7))
    list8xi = manager8xi.list(range(5))
    
    print('=' *33)
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 6/12): CREATED THREE LISTS USING EACH MANAGER!")
    print('=' *33)   
         
        
    # ---------------------------------------------------------------------------------------------
    # (STEP 7/12): SENSE IT!
    #        create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
    
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 7/12): ~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~")
    process2xi = Process2Xi(target=sense, args=('Energy ',queue2xi, pipe2xi_publisher_channel,lock_xi2,dict2xi,list2xi))
    process2xi.start()
    
    print ("PROCESS2XI ALIVE STATUS: ", process2xi.is_alive())       # check whether the process is alive
    print ("PROCESS2XI NAME: ", process2xi.name)                     # the name of the process
    print ("PROCESS2XI PID: ", process2xi.ident)                     # pid = ident of the process
    print ("PROCESS2XI AUTHENTICATION KEY: ", process2xi.authkey)    # authorization key given by the system
    
    print (queue2xi.get())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    print (pipe2xi_consumer_channel.recv())
    
    print("\n DICTIONARY CREATED BY MANAGER2XI AND UPDATED BY PROCESS2XI:")
    print (dict2xi)
    print ("DICT2XI PRESENTATION: ",repr(dict2xi))                   # proxy: prints presentation
    #print ("dict2xi Referent: ", dict2xi.__str__())                 # proxy: return the representation of the referent

    print("\n LIST FOR MANAGER2XI AND PROCESS2XI:")
    print (list2xi)
    # print (list2xi._call_method('__len__'))                        # failed: should call and return the result of a method of the proxy s referent 
    
    process2xi.join()
    print ("PROCESS2XI EXIT CODE AFTER JOIN : ", process2xi.exitcode) # see how the process did exit
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 8/12): REASON IT!
    #             create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
     
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 8/12): ~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~")
    process4xi = Process2Xi(target=reason, args=('Energy ',queue4xi, pipe4xi_publisher_channel,lock_xi4,dict4xi,list8xi))
    process4xi.start()
    
       
    print ("PROCESS4XI ALIVE STATUS: ", process4xi.is_alive())       # check whether the process is alive
    print ("PROCESS4XI NAME: ", process4xi.name)                     # the name of the process
    print ("PROCESS4XI PID: ", process4xi.ident)                     # pid = ident of the process
    print ("PROCESS4XI AUTHENTICATION KEY: ", process4xi.authkey)    # authorization key given by the system
    
    
    print (queue4xi.get())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    print (pipe4xi_consumer_channel.recv())
    
    print("\n DICTIONARY CREATED BY MANAGER4XI AND UPDATED BY PROCESS4XI:")
    print (dict4xi)
    print ("DICT4XI PRESENTATION: ",repr(dict4xi))                   # proxy: prints presentation
    
    print("\n LIST FOR MANAGER4XI AND PROCESS4XI:")
    print (list4xi)
    
    process4xi.join()
    print ("PROCESS4XI EXIT CODE AFTER JOIN : ", process4xi.exitcode) # see how the process did exit
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 9/12): ACE IT!
    #            create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
     
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 9/12): ~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~")
    process8xi = Process2Xi(target=ace, args=('Energy ',queue8xi, pipe8xi_publisher_channel,lock_xi8,dict8xi,list8xi))
    process8xi.start()
    
           
    print ("PROCESS8XI ALIVE STATUS: ", process8xi.is_alive())       # check whether the process is alive
    print ("PROCESS8XI NAME: ", process8xi.name)                     # the name of the process
    print ("PROCESS8XI PID: ", process8xi.ident)                     # pid = ident of the process
    print ("PROCESS8XI AUTHENTICATION KEY: ", process8xi.authkey)    # authorization key given by the system
    
    
    print (queue8xi.get())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    print (pipe8xi_consumer_channel.recv())
    
    print("\n DICTIONARY CREATED BY MANAGER8XI AND UPDATED BY PROCESS8XI:")
    print (dict8xi)
    print ("DICT8XI PRESENTATION: ",repr(dict8xi))                   # proxy: prints presentation
    
    print("\n LIST FOR MANAGER8XI AND PROCESS8XI:")
    print (list8xi)
    
    process8xi.join()
    print ("PROCESS8XI EXIT CODE AFTER JOIN : ", process8xi.exitcode) # see how the process did exit
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 10/12): create three pools with worker processes 
    # ---------------------------------------------------------------------------------------------
    print("\n PHASE1 OF TEST_PROXY_MP_XI4 (STEP 10/12): CREATING THREE POOLS WITH WORKER PROCESSES")
            
    pool2xi = Pool2Xi(processes=2)
    pool4xi = Pool2Xi(processes=4)
    pool8xi = Pool2Xi(processes=8)

    result2xi = pool2xi.apply_async(expxi2, (2,))
    print ("\n (STEP 10/12a): RESULT FROM POOL2XI WORKERS: ",result2xi.get(timeout=1))    
    
    result4xi = pool4xi.apply_async(expxi4, (4,))
    print ("\n (STEP 10/12b): RESULT FROM POOL4XI WORKERS: ",result4xi.get(timeout=1))
    
    result8xi = pool8xi.apply_async(expxi8, (8,))
    print ("\n (STEP 10/12c): RESULT FROM POOL8XI WORKERS: ",result8xi.get(timeout=1))
    
    
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 11/12): explore modifying ctypes objects in shared memory 
    # ---------------------------------------------------------------------------------------------
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 11/12): EXPLORING MODIFYING CTYPES OBJECTS IN SHARED MEMORY")
              
    lock_xi2 = Lock()

    n_xi2 = Value('i', 7)
    x_xi2 = Value(c_double, 1.0/3.0, lock=lock_xi2)
    s_xi2 = Array('c', 'yourDragonXi', lock=lock_xi2)
    A_xi2 = Array(Point4Xi, [(1.875,-6.25), (-5.75,2.0), (2.375,9.5)], lock=lock_xi2)

    # multiprocessing.Array(typecode_or_type, size_or_initializer, *, lock=True)
    #
    # Return a ctypes array allocated from shared memory. By default the return value is actually a synchronized wrapper for the array.
    # typecode_or_type
    #    determines the type of the elements of the returned array: it is either a ctypes type or a one character typecode of the kind used by the array module. 
    # size_or_initializer
    #    If size_or_initializer is an integer, then it determines the length of the array, and the array will be initially zeroed. 
    #    Otherwise, size_or_initializer is a sequence which is used to initialize the array and whose length determines the length of the array.
    # lock
    #    If lock is True (the default) then a new lock object is created to synchronize access to the value. 
    #    If lock is a Lock or RLock object then that will be used to synchronize access to the value. 
    #    If lock is False then access to the returned object will not be automatically protected by a lock, so it will not necessarily be process-safe
    #
    # Notes
    #    lock is a keyword only argument
    #    an array of ctypes.c_char has value and rawvalue attributes which allow one to use it to store and retrieve strings
    #

    # ---------------------------------------------------------------------------------------------
    #
    # ---------------------------------------------------------------------------------------------
    
    process2ctypes = Process2Xi(target=modify2xi, args=(n_xi2, x_xi2, s_xi2, A_xi2))
    process2ctypes.start()
    process2ctypes.join()

    print ("(STEP 11/12a) n_xi2.value: ",n_xi2.value)
    print ("(STEP 11/12b) x_xi2.value: ",x_xi2.value)
    print ("(STEP 11/12c) s_xi2.value: ",s_xi2.value)
    print ([(a.x, a.y) for a in A_xi2])
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 12/12): explore with 3D coordinates
    # ---------------------------------------------------------------------------------------------
    print("\n PHASE2 OF TEST_PROXY_MP_XI4 (STEP 12/12): EXPLORING WITH 3D COORDINATES")
        
    lock_xi3 = Lock2Xi()
    n_xi3 = Value('i', 8)
    x_xi3 = Value(c_double, 1.0/3.0, lock=lock_xi3)
    s_xi3 = Array('c', 'DragonXi3D', lock=lock_xi3)
    A_xi3 = Array(ThreeDimension3Xi, [(1.875,-6.25,2.00), (-5.75,2.0,3.00), (2.375,9.5,4.0)], lock=lock_xi3)
    
    process3xyz = Process2Xi(target=modify3xi, args=(n_xi3, x_xi3, s_xi3, A_xi3))
    process3xyz.start()
    process3xyz.join()
    
    print ("(STEP 12/12a) n_xi3.value: ",n_xi3.value)
    print ("(STEP 12/12b) x_xi3.value: ",x_xi3.value)
    print ("(STEP 12/12c) s_xi3.value: ",s_xi3.value)
    print ([(a.x, a.y, a.z) for a in A_xi3])
    
    print('=' *33)
    print("COMPLETED PHASE 2 - MULTIPROCESSING2XI!")
    print('=' *33)   

#</function4xi>
 
        
# -------------------------------------------------------------------------------------------------
#<function4xi function = "phase4" usage = "
#   using Proxy Objects in dragonized Multiprocessing4Xi 
#"> 
# -------------------------------------------------------------------------------------------------
def phase4():
    global t2xi
    global trace_object_count    
        
    # --------------------------------------------------------------------------------------------- 
    # INITIALIZE TRACE
    # ---------------------------------------------------------------------------------------------        
    t2xi = Print4Xi()
    print("(Step 1/33) OF TEST_PROXY_MP_XI4 - SET T2XI TO: ", t2xi)
    # 
    set2trace4xi()                                                                                               # trace all classes and functions in the system
    deny2trace4xi()                                                                                              # deny trace for all classes and functions in the system
    tailor2trace4xi()                                                                                            # tailor trace for this test   
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 1/12) - OK
    # ---------------------------------------------------------------------------------------------
    print("\n (STEP 1/12A): TRYING TO CREATE THREE QUEUE4XI OBJECTS")
    
    try: 
        queue2xi = Queue4Xi()
        queue4xi = Queue4Xi()
        queue8xi = Queue4Xi() 
        
        print('=' *33)
        print("\n (STEP 1/12B): CREATED THREE QUEUE4XI OBJECTS: ") 
        print('=' *33)
        
        print(queue2xi) 
        print(queue4xi)
        print(queue8xi)
        # <queue4xi.Queue4Xi object at 0x02EEFF90>
        # <queue4xi.Queue4Xi object at 0x02EFF6B0>
        # <queue4xi.Queue4Xi object at 0x02F689D0>
    except:
        print("\n (STEP 1/12C): *** CHALLENGES CREATING THREE QUEUE4XI OBJECTS *** ")          
        
    # ---------------------------------------------------------------------------------------------
    # (STEP 2/12) - OK
    # ---------------------------------------------------------------------------------------------
    print("\n (STEP 2/12A): CREATING MULTIPROCESSING PIPES WITH CONSUMER AND PUBLISHER ENDS")
    
    try:
        pipe2xi_consumer_channel, pipe2xi_publisher_channel = Pipe4Xi() # duplex because parameter not given
        pipe4xi_consumer_channel, pipe4xi_publisher_channel = Pipe4Xi() # duplex
        pipe8xi_consumer_channel, pipe8xi_publisher_channel = Pipe4Xi() # duplex
        
        
        print('=' *33)
        print("\n (STEP 2/12B): CREATED THREE PIPES WITH CONSUMER AND PUBLISHER ENDS: ")
        print('=' *33)
        
        print(pipe2xi_consumer_channel)
        print(pipe2xi_publisher_channel)
        
        print(pipe4xi_consumer_channel)
        print(pipe4xi_publisher_channel)
        
        print(pipe8xi_consumer_channel)
        print(pipe8xi_publisher_channel)
        
        # all pipes are duplex ones:
        #<read-write PipeConnection, handle 356> 
        #<read-write PipeConnection, handle 372>
        
        #<read-write PipeConnection, handle 412>
        #<read-write PipeConnection, handle 396>
        
        #<read-write PipeConnection, handle 428>
        #<read-write PipeConnection, handle 284>

        
             
    except:
        print("\n (Step 2/12C): *** CHALLENGES CREATING THREE PIPES WITH CONSUMER AND PUBLISHER ENDS ***")    
        
    # ---------------------------------------------------------------------------------------------
    # (STEP 3/12) - OK
    # ---------------------------------------------------------------------------------------------   

    print("\n (STEP 3/12A): CREATING THREE LOCKS")      
    try:
        lock_xi2 = Lock4Xi()
        lock_xi4 = Lock4Xi()
        lock_xi8 = Lock4Xi()    
        
        print('=' *33)
        print("\n (STEP 3/12B): CREATED THREE LOCKS:")  
        print('=' *33)
        
        print(lock_xi2)
        print(lock_xi4) 
        print(lock_xi8)     
        
        # <Lock(owner=None)>
        # <Lock(owner=None)>
        # <Lock(owner=None)>
    except:
        print("\n (STEP 3/12C): *** CHALLENGES CREATING THREE LOCKS ***") 
           
 
    
    # ---------------------------------------------------------------------------------------------
    # (STEP 4/12A) - 
    # ---------------------------------------------------------------------------------------------      
           
    print("\n (STEP 4/12A1): CREATING AND STARTING MANAGER TO SENSE")     
    manager2xi_exists = False
    try:
        
        manager2xi = Manager4Xi() 
       
        print('=' *33)
        print("\n (STEP 4/12A2) CREATED AND STARTED MANAGER2XI: ", manager2xi)
        print('=' *33)        
        # <syncmanager4xi.SyncManager4Xi object at 0x0331A1B0>
            
        manager2xi_exists = True
    except:
        print('=' *33)
        print("\n (STEP 4/12A3) *** FAILED TO CREATE AND START MANAGER2XI")
        print('=' *33)
           
    
    # ---------------------------------------------------------------------------------------------
    # STEP 4/12B: OK: CREATED  AND STARTED MANAGER4XI
    # ---------------------------------------------------------------------------------------------      
  
    print("\n 4/12B1): TRYING TO CREATE AND START MANAGER4XI")     
    manager4xi_exists = False
    try:    
        manager4xi = Manager4Xi() 
        
        
        print('=' *33)
        print("\n (STEP 4/12B2): CREATED AND STARTED MANAGER4XI: ", manager4xi)
        print('=' *33)
      
        # <syncmanager4xi.SyncManager4Xi object at 0x033BDEB0>
            
        manager4xi_exists = True
    except:
        print('=' *33)
        print("\n (STEP 4/12B3): *** FAILED TO CREATE  AND START MANAGER4XI")
        print('=' *33)    
    
    # ---------------------------------------------------------------------------------------------
    # STEP 4/12C: OK: CREATED  AND STARTED MANAGER8XI
    # ---------------------------------------------------------------------------------------------      
  
    manager8xi_exists = False    
    print("\n (STEP 4/12C1): TRYING TO CREATE AND START MANAGER8XI")     
    try:
        manager8xi = Manager4Xi() 
        
        
        print('=' *33)
        print("\n (STEP 4/12C2): OK: CREATED  AND STARTED MANAGER8XI: ", manager8xi)
        print('=' *33)
        
        # <syncmanager4xi.SyncManager4Xi object at 0x033C3150> 
    
        manager8xi_exists = True
    except:
        print('=' *33)
        print("\n (STEP 4/12C3): *** FAILED TO CREATE  AND START MANAGER8")
        print('=' *33)        

         
 
            
    # ---------------------------------------------------------------------------------------------
    # STEP 5/12:  TO BE SOLVED - THE LATEST ONE !!!
    #             FAILED TO CREATE THREE DICTIONARIES USING EACH MANAGER
    # ---------------------------------------------------------------------------------------------   
    print("\n (STEP 5/12): TRYING TO CREATE THREE DICTIONARIES USING EACH MANAGER")     
    # 
        
    # ------------------------------------------------------------------------------------ ---------
    # Step 5/12a   
    # ---------------------------------------------------------------------------------------------        
    if manager2xi_exists: 
        print("\n (STEP 5/12A1): TRYING TO CREATE DICT2XI USING MANAGER2XI")
        # 
        # SyncManager4Xi : _create ; (Step 2/5c) asserted; state-value-of-syncmanager2xi-asserted-by-create-of-SyncManager4Xi: 1; time: 16:58:17 03/11/10 FLE Standard Time}
        # Client4Xi : Client4Xi ; (Step 1/6a) started; Client4Xi-started: <function Client4Xi at 0x02E72ED0>; time: 16:58:17 03/11/10 FLE Standard Time}
        # Client4Xi : Client4Xi ; (Step 1/6b) passed; address-passed-to-Client4Xi: None <--- *** FATAL ERROR ***
        
        try:
            dict2xi = manager2xi.dict()
            
            
            print('=' *33)
            print("\n (STEP 5/12A2: CREATED DICT2XI: ", dict2xi) 
            print('=' *33)
            
            
        except:
            print("\n (STEP 5/12A3): *** FAILED TO CREATE DICT2Xi ***") 
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : register ; (Step 8/9a) passing; typeid-passing-to-create-of-syncmanager2xi-by-temp-of-SyncManager4Xi: dict; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 1/5a) started; create-of-SyncManager4Xi-started: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 1/5b) passed; typeid-passed-to-create-of-SyncManager4Xi: dict; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 1/5c) passed; args-passed-to-create-of-SyncManager4Xi: ; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 1/5e) set; StepOriginal-set-by-create-of-SyncManager4Xi to: False; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { get_started_manager4xi : get_started_manager4xi ; started; get-started-manager4xi-started: <function get_started_manager4xi at 0x02B12A50>; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { get_started_manager4xi : get_started_manager4xi ; returning; STARTED-MANAGER4XI-returning-by-get-started-manager4xi: 1; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 2/5a) set; started2xi-set-by-create-of-SyncManager4Xi to: 1; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 2/5b) got; state-value-of-syncmanager2xi-got-by-create-of-SyncManager4Xi: 1; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 2/5c) asserted; state-value-of-syncmanager2xi-asserted-by-create-of-SyncManager4Xi: 1; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 3/5a) passing; address-of-syncmanager2xi-passing-to-Client-of-syncmanager2xi-by-create-of-SyncManager4Xi: None; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 3/5b) yielded; address-of-syncmanager2xi-yielded-by-create-of-SyncManager4Xi: None; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 3/5c) passing; authkey-of-syncmanager2xi-passing-to-Client-of-syncmanager2xi-by-create-of-SyncManager4Xi: b'^KFO\x83b\xe1\xe5\xfc\x98fI\xd7j\x01U\x86\x0e\xc0[\xa1H\xd2+n$\x96eM\xcc\xc6\x08'; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 3/5d) calling; Client-of-syncmanager2xi-calling-by-create-of-SyncManager4Xi: <function Client4Xi at 0x02F1A420>; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 3/5b) yielded; SyncManager4Xi._ack2connection-yielded-by-create-of-SyncManager4Xi: True; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; (Step 4/5i) yielded; dispatch4xi-yielded-by-create-of-SyncManager4Xi: <function dispatch4xi at 0x02EEDE88>; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            # { SyncManager4Xi : _create ; yielded; Token4Xi-yielded-by-create-of-SyncManager4Xi: <class 'token4xi.Token4Xi'>; time: 10:00:15 03/24/10 FLE Standard Time}
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        
            
                   
    else:
        print("\n (STEP 5/12A4): *** SKIPPED BECAUSE MANAGER2XI OBJECT DID NOT EXIST ***")

    # ---------------------------------------------------------------------------------------------
    # STEP 5/12B - OK with original Manager4Xi setting in examine4xi.py   
    # ---------------------------------------------------------------------------------------------            
    if manager4xi_exists:
        print("\n (STEP 5/12B): TRYING TO CREATE DICT4XI USING MANAGER4XI")    
        try:
            dict4xi = manager4xi.dict()
            
            
            print('=' *33)
            print("\n (STEP 5/12B): CREATED DICT4XI: ", dict4xi) 
            print('=' *33)
            # {}
            
        except:
            print("\n (STEP 5/12B): *** FAILED TO CREATE DICT4XI ***")        
    else:
        print("\n (STEP 5/12B): ** SKIPPED BECAUSE MANAGER4XI OBJECT DID NOT EXIST **")
        
    # --------------------------------------------------------------------------------------------- 
    # Step 5/12c - OK with original Manager4Xi setting in examine4xi.py      
    # ---------------------------------------------------------------------------------------------           
    if manager8xi_exists:
        print("\n (STEP 5/12C1): TRYING TO CREATE DICT8XI USING MANAGER8XI")    
        try:
            dict8xi = manager8xi.dict()
            
            
            print('=' *33)
            print("\n (STEP 5/12C2: CREATED DICT8XI: ", dict8xi) 
            print('=' *33)
            # {}
            
        except:
            print("\n (STEP 5/12C3): *** FAILED TO CREATE DICT8XI ***")        
    else:
        print("\n (STEP 5/12C4): ** SKIPPED BECAUSE MANAGER8XI OBJECT DID NOT EXIST **")
                           

        
    # ---------------------------------------------------------------------------------------------
    # Step 6/12:  - OK with original Manager4Xi setting in examine4xi.py    
    #            
    # ---------------------------------------------------------------------------------------------   
    print("\n (STEP 6/12): TRYING TO CREATE THREE LISTS USING EACH MANAGER") 
    # phase2 Step 6/12a: trying to create three lists using each manager
    
    # ---------------------------------------------------------------------------------------------    
    # OK with original Manager4Xi setting in examine4xi.py      
    # ---------------------------------------------------------------------------------------------    
    if manager2xi_exists:
        print("\n (STEP 6/12A1): TRYING TO CREATE LIST2XI USING MANAGER2XI")    
        try:   
            list2xi = manager2xi.list(range(15))
            
            
            print('=' *33)
            print("\n (STEP 6/12A2): CREATED LIST2XI: ", list2xi) 
            print('=' *33)
            # [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
            
        except:
            print("\n (STEP 6/12A3): *** failed to create list2xi ***")         
    else:
        print("\n (STEP 6/12A4): ** SKIPPED BECAUSE MANAGER2XI OBJECT DID NOT EXIST **")            
        
     
    # ---------------------------------------------------------------------------------------------    
    # OK with original Manager4Xi setting in examine4xi.py      
    # ---------------------------------------------------------------------------------------------    
    if manager4xi_exists:
        print("\n (STEP 6/12B1): TRYING TO CREATE LIST4XI USING MANAGER4XI")    
        try:   
            list4xi = manager4xi.list(range(7))
            
            
            print('=' *33)
            print("\n (STEP 6/12B2): CREATED LIST4XI: ", list4xi) 
            print('=' *33)
            # [0, 1, 2, 3, 4, 5, 6]
            
        except:
            print("\n (STEP 6/12B3): *** FAILED TO CREATE LIST4XI ***")         
    else:
        print("\n (STEP 6/12B4): ** SKIPPED BECAUSE MANAGER4XI OBJECT DID NOT EXIST **")            
        
             
     
    # ---------------------------------------------------------------------------------------------    
    # OK with original Manager4Xi setting in examine4xi.py      
    # ---------------------------------------------------------------------------------------------    
    if manager8xi_exists:
        print("\n (STEP 6/12C1): TRYING TO CREATE LIST8XI USING MANAGER8Xi")    
        try:   
            list8xi = manager8xi.list(range(5))
            
            
            print('=' *33)
            print("\n (STEP 6/12C2: CREATED LIST8XI: ", list8xi) 
            print('=' *33)
            # [0, 1, 2, 3, 4]
            
        except:
            print('=' *33)
            print("\n (STEP 6/12C3): *** FAILED TO CREATE LIST8Xi ***")         
            print('=' *33)            
    else:
        print('=' *33)
        print("\n (STEP 6/12C4): ** SKIPPED BECAUSE MANAGER8XI OBJECT DID NOT EXIST ***")            
        print('=' *33)           

            
    # ---------------------------------------------------------------------------------------------
    # Step 7/12: to be solved
    #            SENSE IT! create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
    print("\n (STEP 7/12): STARTING TO SENSE IT WITH YOURDRAGONXI!")    

    # ---------------------------------------------------------------------------------------------    
    #    
    # --------------------------------------------------------------------------------------------- 
       
    if manager2xi_exists:

        # -----------------------------------------------------------------------------------------    
        #    
        # ----------------------------------------------------------------------------------------- 
         
        print('=' *33)    
        print("\n (STEP 7/12A1): TRYING TO CREATE PROCESS2XI")         
        print('=' *33)
                    
        ack2process = False
        nack2process = True

        # -----------------------------------------------------------------------------------------    
        #    
        # ----------------------------------------------------------------------------------------- 
            
        try:
            process2xi = Process4Xi(target=sense, args=('Energy ',queue2xi, pipe2xi_publisher_channel,lock_xi2,dict2xi,list2xi))
            
            print('=' *33)    
            print("\n (STEP 7/12A2): CREATED PROCESS2XI: ", process2xi)         
            print('=' *33)            
            # OK with original Manager4Xi setting in examine4xi.py  
            
            ack2process = True

        except:
            print('=' *33)    
            print("\n (STEP 7/12A3): *** FAILED TO CREATE PROCESS2XI ***")         
            print('=' *33)

        # -----------------------------------------------------------------------------------------    
        #    
        # ----------------------------------------------------------------------------------------- 
                      
        if ack2process is True:

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
              
            print('=' *33)    
            print("\n (STEP 7/12A4): TRYING TO START PROCESS2XI")         
            print('=' *33)            

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                    
            try: 

                # ---------------------------------------------------------------------------------    
                #    
                # --------------------------------------------------------------------------------- 
                                                 
                process2xi.start()
                
                print('=' *33)    
                print("\n (STEP 7/12A5): STARTED PROCESS2XI ", process2xi)         
                print('=' *33) 
                # OK with original Manager4Xi setting in examine4xi.py 
                                  
            except:
            
                print('=' *33)    
                print("\n (STEP 7/12A6): *** FAILED TO START PROCESS2XI ***")         
                print('=' *33)

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12A7): TRYING TO RUN PROCESS2XI")         
            print('=' *33)            
            
            try:                     

                # ---------------------------------------------------------------------------------    
                #    
                # --------------------------------------------------------------------------------- 
                                           
                process2xi.run()
                
                print('=' *33)    
                print("\n (STEP 7/12A8): RUNNING PROCESS2XI", process2xi)          
                print('=' *33)
                # OK with original Manager4Xi setting in examine4xi.py 
                      
            except:
                print('=' *33)    
                print("\n (STEP 7/12A9): *** FAILED TO RUN PROCESS2XI ***")         
                print('=' *33)

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12B): TRYING TO GET PROCESS2XI STATITICS")         
            print('=' *33)            
                                              
            try:
                print ("STEP 7/12C: PROCESS2XI ALIVE STATUS: ", process2xi.is_alive())       # check whether the process is alive
                print ("STEP 7/12D: PROCESS2XI NAME: ", process2xi.name)                     # the name of the process
                print ("STEP 7/12E: PROCESS2XI PID: ", process2xi.ident)                     # pid = ident of the process
                print ("STEP 7/12F: PROCESS2XI AUTHENTICATION KEY: ", process2xi.authkey)    # authorization key given by the system
                # OK with original Manager4Xi setting in examine4xi.py 
                
            except:
                print('=' *33)    
                print("\n (STEP 7/12G): *** FAILED TO GET PROCESS2XI STATITICS")         
                print('=' *33)                
                
            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12H): TRYING TO GET FROM QUEUE2XI")         
            print('=' *33)            
                                              
            try:        
                print (queue2xi.get())
                # OK with original Manager4Xi setting in examine4xi.py                 
                # { Queue4Xi : get ; (Step 14/14b) returning; response-returning-by-get-of-Queue4Xi: ['DragonXi2_00', 'Energy ', 'sensing!']; time: 04:53:51 04/19/10 Pacific Daylight Time}
                # ['DragonXi2_00', 'Energy ', 'sensing!']
            
            except:
                print('=' *33)    
                print("\n (STEP 7/12I): *** FAILED TO GET FROM QUEUE2XI ***")         
                print('=' *33)                  

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12J): TRYING TO RECEIVE FROM PIPE2XI")         
            print('=' *33)            
                                              
            try:                     
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                print (pipe2xi_consumer_channel.recv())
                # OK with original Manager4Xi setting in examine4xi.py                   
                # ['DragonXi2_Sensing_01', 'BIO ', 'Energy ']
                # ['DragonXi2_Sensing_02', 'CO2 ', 'Energy ']
                # ['DragonXi2_Sensing_03', 'Coal ', 'Energy ']
                # ['DragonXi2_Sensing_04', 'Diesel ', 'Energy ']
                # ['DragonXi2_Sensing_05', 'Electricity ', 'Energy ']
                # ['DragonXi2_Sensing_06', 'Fusion ', 'Energy ']
                # ['DragonXi2_Sensing_07', 'Gas ', 'Energy ']
                # ['DragonXi2_Sensing_08', 'Geo ', 'Energy ']
                # ['DragonXi2_Sensing_09', 'Hydrogen ', 'Energy ']
                # ['DragonXi2_Sensing_10', 'Lithium ', 'Energy ']
                # ['DragonXi2_Sensing_11', 'Nuclear Power ', 'Energy ']
                # ['DragonXi2_Sensing_12', 'Ocean ', 'Energy ']
                # ['DragonXi2_Sensing_13', 'Oil ', 'Energy ']
                # ['DragonXi2_Sensing_14', 'Solar ', 'Energy ']
                # ['DragonXi2_Sensing_15', 'Wind ', 'Energy ']
            except:
                        
                print('=' *33)    
                print("\n (STEP 7/12K): *** FAILED TO RECEIVE FROM PIPE2XI ***")         
                print('=' *33)            
                     

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12K): TRYING TO USE DICTIONARY DICT2XI")         
            print('=' *33)            
                                              
            try:                      
                print('=' *33)
                print("\n STEP 7/12L: DICTIONARY CREATED BY MANAGER2XI AND UPDATED BY PROCESS2XI:")
                print (dict2xi)
                print ("STEP 7/12M: DICT2XI PRESENTATION: ",repr(dict2xi))                   # proxy: prints presentation
                print ("STEP 7/12N: DICT2XI REFERENT: ", dict2xi.__str__())                  # proxy: return the representation of the referent
                print('=' *33)
                # OK with original Manager4Xi setting in examine4xi.py                    
                # {1: 'BIO', 2: 'CO2', 3: 'Coal', 4: 'Diesel', 5: 'Electricity', 6: 'Fusion', 7: 'Gas', 8: 'Geo', 9: 'Hydrogen', 10: 'Lithium', 11: 'Nuclear Power', 12: 'Ocean', 13: 'Oil', 14: 'Solar', 15: 'Wind'}
                # DICT2XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x2eb4f50>
                # DICT2XI REFERENT:  {1: 'BIO', 2: 'CO2', 3: 'Coal', 4: 'Diesel', 5: 'Electricity', 6: 'Fusion', 7: 'Gas', 8: 'Geo', 9: 'Hydrogen', 10: 'Lithium', 11: 'Nuclear Power', 12: 'Ocean', 13: 'Oil', 14: 'Solar', 15: 'Wind'}
               
                
            except:            
                print('=' *33)    
                print("\n (STEP 7/12O): *** FAILED TO USE DICTIONARY DICT2XI ***")         
                print('=' *33)            
       
            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12P): TRYING TO USE LIST LIST2XI")         
            print('=' *33)            
                                              
            try:                      
                print('=' *33)
                print("\n STEP 7/12Q: LIST FOR MANAGER2XI AND PROCESS2XI:")                  # [0. 1, ....14]
                print (list2xi)
                print('=' *33)
                # OK with original Manager4Xi setting in examine4xi.py                      
                # [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
            except:            
                print('=' *33)    
                print("\n (STEP 7/12R): *** FAILED TO USE LIST LIST2XI ***")         
                print('=' *33)            
          
            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
                        
            print('=' *33)    
            print("\n (STEP 7/12P): TRYING TO JOIN PROCESS2XI: ", process2xi)         
            print('=' *33)            
                                              
            try:                        
                process2xi.join()
                print('=' *33)
                print ("STEP 7/12R: PROCESS2XI EXIT CODE AFTER JOIN : ", process2xi.exitcode) 
                #  OK with original Manager4Xi setting in examine4xi.py                      
                #  0
            except:
                print('=' *33)    
                print("\n (STEP 7/12S): *** FAILED TO JOIN PROCESS2XI *** ")         
                print('=' *33)      
            
        else:

            # -------------------------------------------------------------------------------------    
            #    
            # ------------------------------------------------------------------------------------- 
        
            print("\n (STEP 7/12): ** SKIPPED - NO PROCESS OBJECT ** ")        
            #
    else:

        # -----------------------------------------------------------------------------------------    
        #    
        # ----------------------------------------------------------------------------------------- 
        
        print("\n (STEP 7/12): ** SKIPPED - NO MANAGER2XI OBJECT  **")            
            
    # ---------------------------------------------------------------------------------------------
    # Step 8/12: - with original Manager4Xi setting in examine4xi.py
    #            REASON IT! create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
    print("\n (STEP 8/12A): TRYING TO REASON IT WITH YOURDRAGONXI!")    
    
    # ---------------------------------------------------------------------------------------------    
    # OK - with original Manager4Xi setting in examine4xi.py   
    # ---------------------------------------------------------------------------------------------    
    if manager4xi_exists:
        print("\n (STEP 8/12A): TRYING WITH MANAGER4XI OBJECT: ", manager4xi)          
        # <multiprocessing.managers.SyncManager object at 0x030AEB30>
        try:
            print("\n (STEP 8/12B): REASON IT WITH YOURDRAGONXI!")
            process4xi = Process4Xi(target=reason, args=('Energy ',queue4xi, pipe4xi_publisher_channel,lock_xi4,dict4xi,list4xi))
            
            process4xi.start()
            
            process4xi.run()
            
            print('=' *33)   
            print ("\n (STEP 8/12C) PROCESS4XI ALIVE STATUS: ", process4xi.is_alive())       # check whether the process is alive
            print ("\n (STEP 8/12D) PROCESS4XI NAME: ", process4xi.name)                     # the name of the process
            print ("\n (STEP 8/12E) PROCESS4XI PID: ", process4xi.ident)                     # pid = ident of the process
            print ("\n (STEP 8/12F) PROCESS4XI AUTHENTICATION KEY: ", process4xi.authkey)    # authorization key given by the system
            
            print("\n (STEP 8/12G): GETTING FROM QUEUE4XI")
            print (queue4xi.get())
            # with original Manager4Xi setting in examine4xi.py
            # ['DragonXi4_00', 'Energy ', 'portfolio must be sustainable!']
            
            
            print("\n (STEP 8/12H): RECEIVING FROM PIPE4XI")            
            print (pipe4xi_consumer_channel.recv())
            print (pipe4xi_consumer_channel.recv())
            print (pipe4xi_consumer_channel.recv())
            print (pipe4xi_consumer_channel.recv())
            print (pipe4xi_consumer_channel.recv())
            print (pipe4xi_consumer_channel.recv())
            print (pipe4xi_consumer_channel.recv())
            # with original Manager4Xi setting in examine4xi.py
            # ['DragonXi4_Reasoning_01', 'BIO ', 'Energy ']
            # ['DragonXi4_Reasoning_02', 'Geo ', 'Energy ']
            # ['DragonXi4_Reasoning_03', 'Hydrogen ', 'Energy ']
            # ['DragonXi4_Reasoning_04', 'Nuclear Power ', 'Energy ']
            # ['DragonXi4_Reasoning_05', 'Ocean ', 'Energy ']
            # ['DragonXi4_Reasoning_06', 'Solar ', 'Energy ']
            # ['DragonXi4_Reasoning_07', 'Wind ', 'Energy ']
            print('=' *33)
            
            print('=' *33)
            print("\n (STEP 8/12I): DICTIONARY CREATED BY MANAGER4XI AND UPDATED BY PROCESS4XI:")
            print (dict4xi)
            # with original Manager4Xi setting in examine4xi.py
            # {1: 'BIO', 2: 'Geo', 3: 'Hydrogen', 4: 'Nuclear Power', 5: 'Ocean', 6: 'Solar', 7: 'Wind'}
            
            print("\n (STEP 8/12J): DICT4XI PRESENTATION: ",repr(dict4xi))                   # proxy: prints presentation
            print('=' *33)
            # with original Manager4Xi setting in examine4xi.py
            # <DictProxy object, typeid 'dict' at 0x30b86f0>
            
            print('=' *33)
            print("\n (STEP 8/12K): LIST FOR MANAGER4XI AND PROCESS4XI:")
            print (list4xi)
            print('=' *33)
            # with original Manager4Xi setting in examine4xi.py
            #[0, 1, 2, 3, 4, 5, 6]
            
            process4xi.join()
            
            print('=' *33)
            print ("\n (STEP 8/12L): PROCESS4XI EXIT CODE AFTER JOIN : ", process4xi.exitcode) 
            print('=' *33)
            # with original Manager4Xi setting in examine4xi.py
            
            print('=' *33)
            print("\n (STEP 8/12M): REASONED IT OK WITH YOURDRAGONXI!")
            # with original Manager4Xi setting in examine4xi.py
            print('=' *33)
        except:
            print("\n (STEP 8/12N): *** CHALLENGES TO REASON WITH YOURDRAGONXI ***")            
    else:
        print("\n (STEP 8/12O): ** SKIPPED BECAUSE MANAGER4XI OBJECT DID NOT EXIST **")            
                        
    # ---------------------------------------------------------------------------------------------
    # Step 9/12: 
    #            ACE IT! create a process and use its services such the content it sends via its queue and pipe    
    # ---------------------------------------------------------------------------------------------
    print("\n (STEP 9/12A): TRYING TO ACE IT WITH YOURDRAGONXI!")  
    
      
    # ---------------------------------------------------------------------------------------------    
    # OK - with original Manager4Xi setting in examine4xi.py   
    # ---------------------------------------------------------------------------------------------    
    if manager8xi_exists:
        print("\n (STEP 9/12A): TRYING WITH MANAGER8XI OBJECT: ", manager8xi)          
        # <multiprocessing.managers.SyncManager object at 0x030ADDD0>
        
        try:
            print("\n (STEP 9/12B): ACE IT WITH YOURDRAGONXI!")
            process8xi = Process4Xi(target=ace, args=('Energy ',queue8xi, pipe8xi_publisher_channel,lock_xi8,dict8xi,list8xi))
            
            
            process8xi.start()
            
            process8xi.run()
            
            print('=' *33)       
            print ("\n (STEP 9/12C): PROCESS8XI ALIVE STATUS: ", process8xi.is_alive())       # check whether the process is alive
            print ("\n (STEP 9/12D): PROCESS8XI NAME: ", process8xi.name)                     # the name of the process
            print ("\n (STEP 9/12E): PROCESS8XI PID: ", process8xi.ident)                     # pid = ident of the process
            print ("\n (STEP 9/12F): PROCESS8XI AUTHENTICATION KEY: ", process8xi.authkey)    # authorization key given by the system
            
            print("\n (STEP 9/12G): GETTING WITH QUEUE8XI: ", queue8xi)
            print (queue8xi.get())
            # ['DragonXi8_00', 'Energy ', 'to be aced!']
            
            print("\n (STEP 9/12H): RECEIVING FROM PIPE8XI")            
            print (pipe8xi_consumer_channel.recv())
            print (pipe8xi_consumer_channel.recv())
            print (pipe8xi_consumer_channel.recv())
            print (pipe8xi_consumer_channel.recv())
            print (pipe8xi_consumer_channel.recv())
            print('=' *33)
            # ['DragonXi8_Ace_01', 'BIO ', 'Energy ']
            # ['DragonXi8_Ace_02', 'Hydrogen ', 'Energy ']
            # ['DragonXi8_Ace_03', 'Nuclear Power ', 'Energy ']
            # ['DragonXi8_Ace_04', 'Solar ', 'Energy ']
            # ['DragonXi8_Ace_05', 'Wind ', 'Energy ']
            
            print('=' *33)
            print("\n (STEP 9/12I): DICTIONARY CREATED BY MANAGER8XI AND UPDATED BY PROCESS8XI:")
            print (dict8xi)
            # {1: 'BIO', 2: 'Hydrogen', 3: 'Nuclear Power', 4: 'Solar', 5: 'Wind'}
            
            print ("\n (STEP 9/12J): DICT8XI PRESENTATION: ",repr(dict8xi))                   # proxy: prints presentation
            print('=' *33)
            # <DictProxy object, typeid 'dict' at 0x30b8710>
            
            print('=' *33)
            print("\n (STEP 9/12K): LIST FOR MANAGER8XI AND PROCESS8XI:")
            print (list8xi)
            print('=' *33)
            # [0, 1, 2, 3, 4]
            
            process8xi.join()
            
            print('=' *33)
            print ("\n (STEP 9/12L): PROCESS8XI EXIT CODE AFTER JOIN : ", process8xi.exitcode) # see how the process did exit
            #
            print("\n (STEP 9/12M): ACED IT OK WITH YOURDRAGONXI!")
            # - with original Manager4Xi setting in examine4xi.py
            
            print('=' *33)
            
        except:
            print("\n (STEP 9/12): TO BE SOLVED: **** FAILED TO ACE IT WITH YOURDRAGONXI ***")        
    else:
        print("\n (STEP 9/12A): TO BE SOLVED: SKIPPED BECAUSE MANAGER8XI OBJECT DID NOT EXIST ***")            
              
    # ---------------------------------------------------------------------------------------------
    # Step 10/12: 
    #             create three pools with worker processes 
    # ---------------------------------------------------------------------------------------------   
    print("\n (STEP 10/12): TRYING TO CREATE THREE POOLS WITH WORKER PROCESSES")    
    
    # ---------------------------------------------------------------------------------------------    
    # Step 10/12a and b:
    # ---------------------------------------------------------------------------------------------    
    pool2xi_exists = False
    try:
        print("\n (STEP 10/12A1): CREATING POOL2XI WITH 2 WORKER PROCESSES")
        pool2xi = Pool4Xi(processes=2)
        pool2xi_exists = True
        
        print('=' *33)
        print("\n (STEP 10/12A2): CREATED POOL2XI: ", pool2xi)
        print('=' *33)
        # <pool4xi.Pool4Xi object at 0x03236B90>
        
    except:
        print("\n (STEP 10/12A3): *** FAILED TO CREATE POOL2XI WITH 2 WORKER PROCESSES ***")        
    
    if pool2xi_exists:
        try:    
            print("\n (STEP 10/12B1): TRYING TO SENSE POOL2XI")                 
            kb2xi = pool2xi.sense()
            
            print('=' *33)
            print ("\n (STEP 10/12B2): SENSED POOL2XI: ") 
            print (kb2xi)
            #  {              'object': <pool4xi.Pool4Xi object at 0x02FA49F0>, 
            #   'taskqueue-of-pool2xi': <queue.Queue object at 0x02FBBB30>, 
            #        'pool-of-pool2xi': [<Process4Xi(PoolWorker4Xi-4, stopped daemon)>, <Process4Xi(PoolWorker4Xi-5, stopped daemon)>],
            #       'cache-of-pool2xi': {}}
   
            print('=' *33)
            
        except:
            print("\n (STEP 10/12B3): *** FAILED TO SENSE POOL2XI ***") 
                  
        try:    
            print("\n (STEP 10/12B4): TRYING WITH POOL2XI RESULT2XI")                
            result2xi = pool2xi.apply_async(expxi2, (2,))
            
            print('=' *33)
            print ("\n (STEP 10/12B5): RESULT FROM POOL2XI WORKERS: ",result2xi.get(timeout=2)) #<--- timeout often here, was 1 first  
            print('=' *33)
            
        except:
            print("\n (STEP 10/12B6): *** FAILED WITH RESULT2XI USING 2 WORKER PROCESSES ***")        
    else:
            print("\n (STEP 10/12B7): ** SKIPPED BECAUSE NO POOL2XI OBJECT **")          
        
    # ---------------------------------------------------------------------------------------------    
    # Step 10/12c and d:
    # ---------------------------------------------------------------------------------------------    
    pool4xi_exists = False
    try:
        print("\n (STEP 10/12C1): CREATING POOL4XI WITH 4 WORKER PROCESSES")
        pool4xi = Pool4Xi(processes=4)
        pool4xi_exists = True
        
        print('=' *33)
        print("\n (STEP 10/12C2): CREATED POOL4XI: ", pool4xi)        
        print('=' *33)
        
    except:
        print("\n (STEP 10/12C3): *** FAILED TO CREATE POOL4XI WITH 4 WORKER PROCESSES ***")        
    
    if pool4xi_exists:
        try:    
            print("\n (STEP 10/12D1): TRYING RESULT4XI")                
            result4xi = pool4xi.apply_async(expxi4, (4,))
            
            print('=' *33)
            print ("\n (STEP 10/12D2): RESULT FROM POOL4XI WORKERs: ",result4xi.get(timeout=2))  
            print('=' *33)
            
        except:
            print("\n (STEP 10/12D3): *** FAILED WITH RESULT4XI USING 4 WORKER PROCESSES ***")        
    else:
        print("\n (STEP 10/12D4): ** SKIPPED BECAUSE NO POOL4XI OBJECt **")          
                      
    # ---------------------------------------------------------------------------------------------    
    # Step 10/12e and f:
    # ---------------------------------------------------------------------------------------------    
    pool8xi_exists = False
    try:
        print("\n (STEP 10/12E1): CREATING POOL8XI WITH 8 WORKER PROCESSES")
        pool8xi = Pool4Xi(processes=8)
        pool8xi_exists = True
        
        print('=' *33)
        print("\n (STEP 10/12E2): CREATED POOL8XI: ", pool8xi)
        print('=' *33)
        
    except:
        print("\n (STEP 10/12E3): *** FAILED TO CREATE POOL8XI WITH 8 WORKER PROCESSES ***")        
    
    if pool8xi_exists:
        try:    
            print("\n (STEP 10/12F1): TRYING RESULT8XI")                
            result4xi = pool4xi.apply_async(expxi8, (8,))
            
            print('=' *33)
            print ("\n (STEP 10/12F2): RESULT FROM POOL8XI WORKERS: ",result8xi.get(timeout=2))  
            print('=' *33)
            
        except:
            print("\n (STEP 10/12F3): *** FAILED WITH RESULT8XI USING 8 WORKER PROCESSES ***")        
    else:
            print("\n (STEP 10/12F4): ** SKIPPED BECAUSE NO POOL8XI OBJECT **")                
 
     
                    
    # ---------------------------------------------------------------------------------------------
    # Step 11/12: 
    # ---------------------------------------------------------------------------------------------  
    print("\n (STEP 11/12A1): TRYING TO CREATE LOCK_XI2 WITH LOCK4XI")    
    try:
      

        lock_xi2 = Lock4Xi()
        
        print('=' *33)
        print("\n (STEP 11/12A2: CREATED LOCK_XI2: ")
        print(lock_xi2)
        print('=' *33)
        
        #  
        
    except:
        print("\n (STEP 11/12A-B): ** FAILED WITH LOCK4XI **")          
    
    # -----------------------------------------------------------------------------------------    
    # Step 11/12: c and d
    # -----------------------------------------------------------------------------------------        
    print("\n (STEP 11/12C): TRYING TO CREATE N_XI2 WITH VALUE4XI")    
    try:        

        n_xi2 = Value4Xi('i', 7)
        print("\n (STEP 11/12D1): CREATED N_XI2 WITH VALUE4XI")        
        print("n_xi2: ", n_xi2)

        
        x_xi2 = Value4Xi(c_double, 1.0/3.0, lock=lock_xi2)
        
        print('=' *33)
        print("\n (STEP 11/12D2): CREATED X_XI2 WITH VALUE4XI")           
        print("x_xi2: ", x_xi2)
        # x_xi2:  Value4Xi(<class 'ctypes.c_double'>, 0.3333333333333333)
        
        print('=' *33)
                
    except:
        print("\n (STEP 11/12C-D): TO BE SOLVED: *** FAILED WITH VALUE4XI ***")  
    
    # -----------------------------------------------------------------------------------------    
    # Step 11/12: e  
    # -----------------------------------------------------------------------------------------        
    print("\n (STEP 11/12E): TRYING TO CREATE S_XI2 WITH ARRAYFUNCTION4XI")    
    try:                    

        s_xi2 = ArrayFunction4Xi('c', 'yourDragonXi', lock=lock_xi2)
        
        print('=' *33)
        print("\n (STEP 11/12E): CREATED S_XI2 WITH ARRAYFUNCTION4XI;")        
        print("s_xi2: ", s_xi2)
        print('=' *33)
        
        # phase2 Step 11/12e: created s_xi2 with ArrayFunction4Xi
       
    except:
        print("\n (STEP 11/12E EXCEPTION): *** FAILED WITH ARRAYFUNCTION4XI ***") 

         
    # -----------------------------------------------------------------------------------------    
    # Step 11/12f: OK
    # -----------------------------------------------------------------------------------------        
    print("\n (STEP 11/12F): TRY TO CREATE A_XI2 WITH ARRAYFUNCTION4XI")            
    try:
        
        A_xi2 = ArrayFunction4Xi(Point4Xi, [(1.875,-6.25), (-5.75,2.0), (2.375,9.5)], lock=lock_xi2)
        
        print('=' *33)
        print("\n (STEP 11/12F): CREATED A_XI2 WITH ARRAYFUNCTION4XI:")        
        print("A_xi2: ", A_xi2)
        print('=' *33)

        
    except:
        print("\n (STEP 11/12F): TO BE SOLVED: *** FAILED WITH ARRAYFUNCTION4XI ***")  
    
    # -------------------------------------------------------------------------------------------------    
    # Step 11/12g: OK 
    # -------------------------------------------------------------------------------------------------        
    print("\n (STEP 11/12G1): TRY TO CALL PROCESS4XI GIVING MODIFY2XI FUNCTION AS TARGET AND CREATED CALCULATIONS AS ARGUMENTS")                 
    
    try:
        
        
        process2ctypes = Process4Xi(target=modify2xi, args=(n_xi2, x_xi2, s_xi2, A_xi2))
        
        print('=' *33)
        print("\n (STEP 11/12G2): PERFORMED PROCESS4XI GIVING MODIFY2XI FUNCTION AS TARGET AND CREATED CALCULATIONS AS ARGUMENTS: ")
                
        print("process2ctypes: ", process2ctypes)
        print('=' *33)
        

        process2ctypes.start()
        
        print('=' *33)
        print("\n (STEP 11/12G3): PERFORMED START")          
        print('=' *33)

        process2ctypes.run()
        
        print('=' *33)
        print("\n (STEP 11/12G4): PERFORMED RUN")          
        print('=' *33)

        
        
                
        process2ctypes.join()
        
        print('=' *33)
        print("\n (STEP 11/12G5): PERFORMED JOIN")           

        print('=' *33)
        
    except:
        print("\n (STEP 11/12G6): *** FAILED WITH PROCESS4XI ***")  

   
            
    # -----------------------------------------------------------------------------------------    
    # Step 11/12h: 
    # -----------------------------------------------------------------------------------------        
    print("\n (STEP 11/12H1): TRYING TO PRINT VALUES GIVEN BY PROCESS4XI ")     
    try:          
        print('=' *33)
        print ("\n (STEP 11/12H1) N_XI2.VALUE: ",n_xi2.value)
        print('=' *33)
                
    except:
        print("\n (STEP 11/12H2): TO BE SOLVED: *** FAILED TO PRINT N_XI2.VALUE ***")  
    
    # -----------------------------------------------------------------------------------------    
    # Step 11/12i: 
    # -----------------------------------------------------------------------------------------                 
    try:
        print('=' *33)
        print ("\n (STEP 11/12H3): x_xi2.value: ",x_xi2.value)
        print('=' *33)
        
    except:
        print("\n (STEP 11/12I): *** FAILED TO PRINT X_XI2.VALUE ***") 
         
    # -----------------------------------------------------------------------------------------    
    # Step 11/12j: to be solved 
    # -----------------------------------------------------------------------------------------     
    try:
        print('=' *33)            
        print ("\n (STEP 11/12J1): S_XI2.VALUE: ",s_xi2.value)
        print('=' *33)
        
    except:
        print("\n (STEP 11/12J2): *** FAILED TO PRINT S_XI2.VALUE ***") 
         
    # -----------------------------------------------------------------------------------------    
    # Step 11/12k: 
    # ----------------------------------------------------------------------------------------- 
    try:            
        print('=' *33)
        print("\n (STEP 11/12K1): PRINTED A_XI2 VALUES:")
        print ([(a.x, a.y) for a in A_xi2])
        
        print('=' *33)
        
    except:
        print("\n (STEP 11/12K2): *** FAILED TO PRINT A_XI2 VALUES ***")          
    
     

               
    # ---------------------------------------------------------------------------------------------
    # Step 12/12): OK
    # ---------------------------------------------------------------------------------------------    
    print("\n (STEP 12/12A): CREATING LOCK TO EXPLORE 3D COORDINATES")
        
    
    try:
        lock_xi3 = Lock4Xi()
        
        print('=' *33)
        print("\n (STEP 12/12B): CREATED LOCK_XI3: ", str(lock_xi3)) 
        print('=' *33)

    except:
        print("\n (STEP 12/12B): EXCEPTION *** FAILED IN CALLING LOCK4XI ***")
     
    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12C) 
    # -----------------------------------------------------------------------------------------                 
    print("\n (STEP 12/12C1): TRY TO CREATE N_XI3: ")        
    try:
        n_xi3 = Value4Xi('i', 8)
        
        print('=' *33)
        print("\n (STEP 12/12C2): CREATED N_XI3: ", str(n_xi3))
        print('=' *33)

    except:
        print("\n (STEP 12/12C3): *** FAILED IN CALLING VALUE4Xi ***")

    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12D) 
    # -----------------------------------------------------------------------------------------                 
    print("\n (STEP 12/12D1): CREATING X_XI3: ")            
    try:        
        x_xi3 = Value4Xi(c_double, 1.0/3.0, lock=lock_xi3)
        
        
        print('=' *33)
        print("\n (STEP 12/12D2): CREATED X_XI3: ", str(x_xi3))
        print('=' *33)

    except:
        print("\n (STEP 12/12D3): *** FAILED IN CALLING VALUE4XI ***")
                    

    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12E)
    # -----------------------------------------------------------------------------------------                 
    print("\n (STEP 12/12E1): CREATING S_XI3: ")     
    try:    
        s_xi3 = ArrayFunction4Xi('c', 'DragonXi3D', lock=lock_xi3)
        
        
        print('=' *33)
        print("\n (STEP 12/12E2): CREATED S_XI3: ", str(s_xi3))
        print('=' *33)

    except:
        print("\n (STEP 12/12E): *** failed in calling ArrayFunction4Xi ***")            

     
    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12F) 
    # -----------------------------------------------------------------------------------------                 
    print("\n (STEP 12/12F1): CREATING A_XI3")                
    try:
        A_xi3 = ArrayFunction4Xi(ThreeDimension3Xi, [(1.875,-6.25,2.00), (-5.75,2.0,3.00), (2.375,9.5,4.0)], lock=lock_xi3)
        
        
        print('=' *33)
        print("\n (STEP 12/12F2): CREATED A_XI3: ", str(A_xi3))
        print('=' *33)

         
    except:
        print("\n (STEP 12/12F): *** FAILED IN CALLING ARRAYFUNCTION4XI FOR 3D ***")         

    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12G):
    # -----------------------------------------------------------------------------------------                 
    print("\n (STEP 12/12G1): CREATING PROCESS3XYZ OBJECT FROM PROCESS4XI")     
    
    try:
        process3xyz = Process4Xi(target=modify3xi, args=(n_xi3, x_xi3, s_xi3, A_xi3))
        
        
        print('=' *33)
        print("\n (STEP 12/12G2): CREATED PROCESS3XYZ: ", process3xyz) 
        print('=' *33)

        process3xyz.start()
        print('=' *33)
        print("\n (STEP 12/12G3): STARTED PROCESS3XYZ: ", process3xyz) 
        print('=' *33)
        
        process3xyz.run()        
        print('=' *33)
        print("\n (STEP 12/12G4): RUNNING PROCESS3XYZ: ", process3xyz) 
        print('=' *33)

        process3xyz.join()
        
        
        print('=' *33)
        print("\n (STEP 12/12G5: JOINED PROCESS3XYZ: ", process3xyz) 
        print('=' *33)


    except:
        print("\n (STEP 12/12G): *** FAILED WITH PROCESS3XYZ OBJECT OF PROCESS4XI ***")
        
    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12H): 
    # -----------------------------------------------------------------------------------------                          
    print("\n (STEP 12/12H1): STARTING TO PRINT VALUES USING CREATED OBJECTS")


    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12I):
    # -----------------------------------------------------------------------------------------             
    try:        
        print ("\n (STEP 12/12I1): N_XI3.VALUE: ",n_xi3.value)

    except:
        print("\n (STEP 12/12I2): *** FAILED USING N_XI3 OBJECt")

        
    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12J):
    # -----------------------------------------------------------------------------------------             
    print ("\n (STEP 12/12J1): TRYING X_XI3.VALUE")
    try:        
        print ("\n (STEP 12/12J2): X_XI3.VALUE: ",x_xi3.value)

    except:
        print ("\n (STEP 12/12J3): *** FAILED WITH X_XI3.VALUE ***")
        
    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12K)
    # -----------------------------------------------------------------------------------------             
    print('=' *33)
    print("\n (STEP 12/12K1): TRYING S_XI3.VALUE")
    print('=' *33)        
    try:        
        print ("\n (STEP 12/12K2):S_XI3.VALUE: ",s_xi3.value)

    except:
        print("\n (STEP 12/12K2):*** FAILED USING S_XI3 OBJECT ***")
        
        
    # -----------------------------------------------------------------------------------------    
    # (STEP 12/12L)
    # -----------------------------------------------------------------------------------------             
    print('=' *33)
    print("\n (STEP 12/12L1): TRYING 3D COORDINATES WITH  A_XI3 OBJECT")
    print('=' *33)    
    
    try: 
        print('=' *33)
        print("\n (STEP 12/12L2): 3D COORDINATES WITH  A_XI3 OBJECT: ")
        print('=' *33)
                   
        print ([(a.x, a.y, a.z) for a in A_xi3])

                    
    except:
        print("\n (STEP 12/12L3): *** FAILED USING A_XI3 OBJECT")        
        
        
        
                        
    # ---------------------------------------------------------------------------------------------    
    # PRINT TRACE
    # ---------------------------------------------------------------------------------------------     
    print('=' *33)
    print("\n (STEP 12/12M): CALLING PRINT_TRACE4XI WITH T2XI: ", str(t2xi))
    print('=' *33)
    
    print_trace4xi(t2xi)           

    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------                 
    print('=' *33)
    print("\n PHASE 4 DONE! ")
    print('=' *33) 
#</function4xi>
 

# -------------------------------------------------------------------------------------------------
# main function                              
#<function4xi function = "main" usage = "
#   main of test_proxy_mp_xi4
#"> 
# -------------------------------------------------------------------------------------------------
def main():
    global t2xi 
    
    # phase1()   # Multiprocessing   
    
    # phase2()   # Multiprocessing2Xi  
    
    phase4()   # Multiprocessing4Xi - tune examine4xi to use original multiprocessing modules 


            
#</function4xi>
  

# -------------------------------------------------------------------------------------------------
#                               
# -------------------------------------------------------------------------------------------------
if __name__ == '__main__':
    main()

#</program4xi>

# -------------------------------------------------------------------------------------------------
# History of test_proxy_mp_xi4.py                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
# test_proxy_mp_xi4.txt
# sense-proxy-mp-test4xi.xml
# reason-proxy-mp-test4xi.xml
# ace-proxy-mp-test4xi.xml


# =================================
# 
#  PHASE1 OF TEST_PROXY_MP_XI4 (STEP 6/12): CREATED THREE LISTS USING EACH MANAGER!
# =================================
# 
# PHASE1 OF TEST_PROXY_MP_XI4 (STEP 7/12): ~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~
# PROCESS2XI ALIVE STATUS:  True
# PROCESS2XI NAME:  Process-4
# PROCESS2XI PID:  1932
# PROCESS2XI AUTHENTICATION KEY:  b'\x92*\xb1\x9d\x124\x07"_\x14\x1f\xd9i\x7f\xfb\xaeQ\xa6-\x97\xa2\xf2\x80P\xc3~\x08\x04\xcc\xeamv'
# ['DragonXi2_00', 'Energy ', 'sensing!']
# ['DragonXi2_Sensing_01', 'BIO ', 'Energy ']
# ['DragonXi2_Sensing_02', 'CO2 ', 'Energy ']
# ['DragonXi2_Sensing_03', 'Coal ', 'Energy ']
# ['DragonXi2_Sensing_04', 'Diesel ', 'Energy ']
# ['DragonXi2_Sensing_05', 'Electricity ', 'Energy ']
# ['DragonXi2_Sensing_06', 'Fusion ', 'Energy ']
# ['DragonXi2_Sensing_07', 'Gas ', 'Energy ']
# ['DragonXi2_Sensing_08', 'Geo ', 'Energy ']
# ['DragonXi2_Sensing_09', 'Hydrogen ', 'Energy ']
# ['DragonXi2_Sensing_10', 'Lithium ', 'Energy ']
# ['DragonXi2_Sensing_11', 'Nuclear Power ', 'Energy ']
# ['DragonXi2_Sensing_12', 'Ocean ', 'Energy ']
# ['DragonXi2_Sensing_13', 'Oil ', 'Energy ']
# ['DragonXi2_Sensing_14', 'Solar ', 'Energy ']
# ['DragonXi2_Sensing_15', 'Wind ', 'Energy ']
# 
# DICTIONARY CREATED BY MANAGER2XI AND UPDATED BY PROCESS2XI:
# {1: 'BIO', 2: 'CO2', 3: 'Coal', 4: 'Diesel', 5: 'Electricity', 6: 'Fusion'}
# DICT2XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x305ca70>
# 
# LIST FOR MANAGER2XI AND PROCESS2XI:
# [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
# PROCESS2XI EXIT CODE AFTER JOIN :  0
# 
# PHASE1 OF TEST_PROXY_MP_XI4 (STEP 8/12): ~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~
# PROCESS4XI ALIVE STATUS:  True
# PROCESS4XI NAME:  Process-5
# PROCESS4XI PID:  4508
# PROCESS4XI AUTHENTICATION KEY:  b'\x92*\xb1\x9d\x124\x07"_\x14\x1f\xd9i\x7f\xfb\xaeQ\xa6-\x97\xa2\xf2\x80P\xc3~\x08\x04\xcc\xeamv'
# ['DragonXi4_00', 'Energy ', 'portfolio must be sustainable!']
# ['DragonXi4_Reasoning_01', 'BIO ', 'Energy ']
# ['DragonXi4_Reasoning_02', 'Geo ', 'Energy ']
# ['DragonXi4_Reasoning_03', 'Hydrogen ', 'Energy ']
# ['DragonXi4_Reasoning_04', 'Nuclear Power ', 'Energy ']
# ['DragonXi4_Reasoning_05', 'Ocean ', 'Energy ']
# ['DragonXi4_Reasoning_06', 'Solar ', 'Energy ']
# ['DragonXi4_Reasoning_07', 'Wind ', 'Energy ']
# 
# DICTIONARY CREATED BY MANAGER4XI AND UPDATED BY PROCESS4XI:
# {1: 'BIO', 2: 'Geo', 3: 'Hydrogen', 4: 'Nuclear Power', 5: 'Ocean', 6: 'Solar', 7: 'Wind'}
# DICT4XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x305c350>
# 
# LIST FOR MANAGER4XI AND PROCESS4XI:
# [0, 1, 2, 3, 4, 5, 6]
# PROCESS4XI EXIT CODE AFTER JOIN :  0
# 
# PHASE1 OF TEST_PROXY_MP_XI4 (STEP 9/12): ~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~
# PROCESS8XI ALIVE STATUS:  True
# PROCESS8XI NAME:  Process-6
# PROCESS8XI PID:  1060
# PROCESS8XI AUTHENTICATION KEY:  b'\x92*\xb1\x9d\x124\x07"_\x14\x1f\xd9i\x7f\xfb\xaeQ\xa6-\x97\xa2\xf2\x80P\xc3~\x08\x04\xcc\xeamv'
# ['DragonXi8_00', 'Energy ', 'to be aced!']
# ['DragonXi8_Ace_01', 'BIO ', 'Energy ']
# ['DragonXi8_Ace_02', 'Hydrogen ', 'Energy ']
# ['DragonXi8_Ace_03', 'Nuclear Power ', 'Energy ']
# ['DragonXi8_Ace_04', 'Solar ', 'Energy ']
# ['DragonXi8_Ace_05', 'Wind ', 'Energy ']
# 
# DICTIONARY CREATED BY MANAGER8XI AND UPDATED BY PROCESS8XI:
# {1: 'BIO', 2: 'Hydrogen', 3: 'Nuclear Power', 4: 'Solar', 5: 'Wind'}
# DICT8XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x305ca90>
# 
# LIST FOR MANAGER8XI AND PROCESS8XI:
# [0, 1, 2, 3, 4]
# PROCESS8XI EXIT CODE AFTER JOIN :  0
# 
# PHASE1 OF TEST_PROXY_MP_XI4 (STEP 10/12): CREATING THREE POOLS WITH WORKER PROCESSES
# 
# (STEP 10/12a): RESULT FROM POOL2XI WORKERS:  4
# 
# (STEP 10/12b): RESULT FROM POOL4XI WORKERS:  256
# 
# (STEP 10/12c): RESULT FROM POOL8XI WORKERS:  16777216
# 
# PHASE1 OF TEST_PROXY_MP_XI4 (STEP 11/12): EXPLORING MODIFYING CTYPES OBJECTS IN SHARED MEMORY
# (STEP 11/12a) n_xi2.value:  49
# (STEP 11/12b) x_xi2.value:  0.111111111111
# (STEP 11/12c) s_xi2.value:  b'YOURDRAGONXI'
# [(3.515625, 39.0625), (33.0625, 4.0), (5.640625, 90.25)]
# 
# PHASE1 OF TEST_PROXY_MP_XI4 (STEP 12/12): EXPLORING WITH 3D COORDINATES
# (STEP 12/12a) n_xi3.value:  512
# (STEP 12/12b) x_xi3.value:  0.037037037037
# (STEP 12/12c) s_xi3.value:  b'DRAGONXI3D'
# [(6.591796875, -244.140625, 8.0), (-190.109375, 8.0, 27.0), (13.396484375, 857.375, 64.0)]
# =================================
# COMPLETED PHASE 1 - ORIGINAL MULTIPROCESSING!
# =================================
# 
# 
# =================================
# STARTING PHASE 2!
# =================================
# =================================
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 1/12): CREATED MULTIPROCESSING QUEUES!
# =================================
# =================================
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 2/12): CREATED MULTIPROCESSING PIPES WITH CONSUMER AND PUBLISHER ENDS!
# =================================
# =================================
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 3/12): CREATED THREE LOCKS!
# =================================
# =================================
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 4/12): CREATED THREE MANAGERS!
# =================================
# =================================
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 5/12): CREATED THREE DICTIONARIES USING EACH MANAGER!
# =================================
# =================================
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 6/12): CREATED THREE LISTS USING EACH MANAGER!
# =================================
# 
#  PHASE1 OF TEST_PROXY_MP_XI4 (STEP 7/12): ~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~2~
# PROCESS2XI ALIVE STATUS:  True
# PROCESS2XI NAME:  Process2Xi-26
# PROCESS2XI PID:  4304
# PROCESS2XI AUTHENTICATION KEY:  b'\x92*\xb1\x9d\x124\x07"_\x14\x1f\xd9i\x7f\xfb\xaeQ\xa6-\x97\xa2\xf2\x80P\xc3~\x08\x04\xcc\xeamv'
# ['DragonXi2_00', 'Energy ', 'sensing!']
# ['DragonXi2_Sensing_01', 'BIO ', 'Energy ']
# ['DragonXi2_Sensing_02', 'CO2 ', 'Energy ']
# ['DragonXi2_Sensing_03', 'Coal ', 'Energy ']
# ['DragonXi2_Sensing_04', 'Diesel ', 'Energy ']
# ['DragonXi2_Sensing_05', 'Electricity ', 'Energy ']
# ['DragonXi2_Sensing_06', 'Fusion ', 'Energy ']
# ['DragonXi2_Sensing_07', 'Gas ', 'Energy ']
# ['DragonXi2_Sensing_08', 'Geo ', 'Energy ']
# ['DragonXi2_Sensing_09', 'Hydrogen ', 'Energy ']
# ['DragonXi2_Sensing_10', 'Lithium ', 'Energy ']
# ['DragonXi2_Sensing_11', 'Nuclear Power ', 'Energy ']
# ['DragonXi2_Sensing_12', 'Ocean ', 'Energy ']
# ['DragonXi2_Sensing_13', 'Oil ', 'Energy ']
# ['DragonXi2_Sensing_14', 'Solar ', 'Energy ']
# ['DragonXi2_Sensing_15', 'Wind ', 'Energy ']
# 
#  DICTIONARY CREATED BY MANAGER2XI AND UPDATED BY PROCESS2XI:
# {1: 'BIO', 2: 'CO2', 3: 'Coal', 4: 'Diesel', 5: 'Electricity', 6: 'Fusion'}
# DICT2XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x3063dd0>
# 
#  LIST FOR MANAGER2XI AND PROCESS2XI:
# [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]
# PROCESS2XI EXIT CODE AFTER JOIN :  0
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 8/12): ~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~4~
# PROCESS4XI ALIVE STATUS:  True
# PROCESS4XI NAME:  Process2Xi-27
# PROCESS4XI PID:  4584
# PROCESS4XI AUTHENTICATION KEY:  b'\x92*\xb1\x9d\x124\x07"_\x14\x1f\xd9i\x7f\xfb\xaeQ\xa6-\x97\xa2\xf2\x80P\xc3~\x08\x04\xcc\xeamv'
# ['DragonXi4_00', 'Energy ', 'portfolio must be sustainable!']
# ['DragonXi4_Reasoning_01', 'BIO ', 'Energy ']
# ['DragonXi4_Reasoning_02', 'Geo ', 'Energy ']
# ['DragonXi4_Reasoning_03', 'Hydrogen ', 'Energy ']
# ['DragonXi4_Reasoning_04', 'Nuclear Power ', 'Energy ']
# ['DragonXi4_Reasoning_05', 'Ocean ', 'Energy ']
# ['DragonXi4_Reasoning_06', 'Solar ', 'Energy ']
# ['DragonXi4_Reasoning_07', 'Wind ', 'Energy ']
# 
#  DICTIONARY CREATED BY MANAGER4XI AND UPDATED BY PROCESS4XI:
# {1: 'BIO', 2: 'Geo', 3: 'Hydrogen', 4: 'Nuclear Power'}
# DICT4XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x3063c50>
# 
#  LIST FOR MANAGER4XI AND PROCESS4XI:
# [0, 1, 2, 3, 4, 5, 6]
# PROCESS4XI EXIT CODE AFTER JOIN :  0
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 9/12): ~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~8~
# PROCESS8XI ALIVE STATUS:  True
# PROCESS8XI NAME:  Process2Xi-28
# PROCESS8XI PID:  4568
# PROCESS8XI AUTHENTICATION KEY:  b'\x92*\xb1\x9d\x124\x07"_\x14\x1f\xd9i\x7f\xfb\xaeQ\xa6-\x97\xa2\xf2\x80P\xc3~\x08\x04\xcc\xeamv'
# ['DragonXi8_00', 'Energy ', 'to be aced!']
# ['DragonXi8_Ace_01', 'BIO ', 'Energy ']
# ['DragonXi8_Ace_02', 'Hydrogen ', 'Energy ']
# ['DragonXi8_Ace_03', 'Nuclear Power ', 'Energy ']
# ['DragonXi8_Ace_04', 'Solar ', 'Energy ']
# ['DragonXi8_Ace_05', 'Wind ', 'Energy ']
# 
#  DICTIONARY CREATED BY MANAGER8XI AND UPDATED BY PROCESS8XI:
# {1: 'BIO', 2: 'Hydrogen', 3: 'Nuclear Power', 4: 'Solar', 5: 'Wind'}
# DICT8XI PRESENTATION:  <DictProxy object, typeid 'dict' at 0x3063cf0>
# 
#  LIST FOR MANAGER8XI AND PROCESS8XI:
# [0, 1, 2, 3, 4]
# PROCESS8XI EXIT CODE AFTER JOIN :  0
# 
#  PHASE1 OF TEST_PROXY_MP_XI4 (STEP 10/12): CREATING THREE POOLS WITH WORKER PROCESSES
# 
#  (STEP 10/12a): RESULT FROM POOL2XI WORKERS:  4
# 
#  (STEP 10/12b): RESULT FROM POOL4XI WORKERS:  256
# 
#  (STEP 10/12c): RESULT FROM POOL8XI WORKERS:  16777216
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 11/12): EXPLORING MODIFYING CTYPES OBJECTS IN SHARED MEMORY
# (STEP 11/12a) n_xi2.value:  49
# (STEP 11/12b) x_xi2.value:  0.111111111111
# (STEP 11/12c) s_xi2.value:  b'YOURDRAGONXI'
# [(3.515625, 39.0625), (33.0625, 4.0), (5.640625, 90.25)]
# 
#  PHASE2 OF TEST_PROXY_MP_XI4 (STEP 12/12): EXPLORING WITH 3D COORDINATES
# (STEP 12/12a) n_xi3.value:  512
# (STEP 12/12b) x_xi3.value:  0.037037037037
# (STEP 12/12c) s_xi3.value:  b'DRAGONXI3D'
# [(6.591796875, -244.140625, 8.0), (-190.109375, 8.0, 27.0), (13.396484375, 857.375, 64.0)]
# =================================
# COMPLETED PHASE 2 - MULTIPROCESSING2XI!
# =================================
# 
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of test_proxy_mp_xi4.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
